@@ -1,5 +1,5 @@
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -25,30 +25,37 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <syslog.h>
 #include <time.h>
 #include <unistd.h>
+#include <algorithm>
+#include <list>
 
 #include <list>
 
 #include "chunkserver/bgjobs.h"
-#include "chunkserver/csserv.h"
 #include "chunkserver/hddspacemgr.h"
+#include "chunkserver/network_main_thread.h"
 #include "common/cfg.h"
-#include "common/cstoma_communication.h"
 #include "common/datapack.h"
 #include "common/goal.h"
+#include "common/loop_watchdog.h"
 #include "common/main.h"
 #include "common/massert.h"
-#include "common/MFSCommunication.h"
 #include "common/moosefs_vector.h"
-#include "common/packet.h"
+#include "common/output_packet.h"
 #include "common/random.h"
 #include "common/slogger.h"
 #include "common/sockets.h"
 #include "common/time_utils.h"
+#include "devtools/request_log.h"
+#include "protocol/cstoma.h"
+#include "protocol/input_packet.h"
+#include "protocol/matocs.h"
+#include "protocol/MFSCommunication.h"
+#include "protocol/packet.h"
 
 #define MaxPacketSize 10000
 
@@ -60,30 +67,16 @@
 #define BGJOBSCNT 1000
 
 // mode
-enum {FREE,CONNECTING,HEADER,DATA,KILL};
-
-struct InputPacket {
-	uint8_t *startptr;
-	uint32_t bytesLeft;
-	std::vector<uint8_t> packet;
-	InputPacket() : startptr(NULL), bytesLeft(0) {}
-};
-
-struct OutputPacket {
-	std::vector<uint8_t> packet;
-	uint32_t bytesSent;
-
-	OutputPacket() : bytesSent(0) {
-	};
-};
+enum {FREE,CONNECTING,CONNECTED,KILL};
 
 struct masterconn {
+	masterconn() : inputPacket(MaxPacketSize) {}
+
 	int mode;
 	int sock;
 	int32_t pdescpos;
 	Timer lastread,lastwrite;
-	uint8_t hdrbuff[PacketHeader::kSize];
-	InputPacket inputpacket;
+	InputPacket inputPacket;
 	std::list<OutputPacket> outputPackets;
 	uint32_t bindip;
 	uint32_t masterip;
@@ -121,12 +114,7 @@ void masterconn_stats(uint64_t *bin,uint64_t *bout,uint32_t *maxjobscnt) {
 }
 
 void* masterconn_create_detached_packet(uint32_t type,uint32_t size) {
-	OutputPacket* outpacket = new OutputPacket();
-	outpacket->packet.resize(size + PacketHeader::kSize);
-	uint8_t *ptr = outpacket->packet.data();
-	put32bit(&ptr, type);
-	put32bit(&ptr, size);
-	return outpacket;
+	return new OutputPacket(PacketHeader(type, size));
 }
 
 uint8_t* masterconn_get_packet_data(void *packet) {
@@ -145,9 +133,8 @@ void masterconn_attach_packet(masterconn *eptr, void* packet) {
 	delete outputPacket;
 }
 
-void masterconn_create_attached_packet(masterconn *eptr, std::vector<uint8_t> serializedPacket) {
-	eptr->outputPackets.emplace_back();
-	eptr->outputPackets.back().packet = std::move(serializedPacket);
+void masterconn_create_attached_packet(masterconn *eptr, MessageBuffer serializedPacket) {
+	eptr->outputPackets.emplace_back(std::move(serializedPacket));
 }
 
 template<class... Data>
@@ -159,7 +146,7 @@ void masterconn_create_attached_moosefs_packet(masterconn *eptr,
 }
 
 void masterconn_sendregisterlabel(masterconn *eptr) {
-	if (eptr->mode == HEADER || eptr->mode == DATA) {
+	if (eptr->mode == CONNECTED) {
 		masterconn_create_attached_packet(eptr, cstoma::registerLabel::build(gLabel));
 	}
 }
@@ -171,18 +158,27 @@ void masterconn_sendregister(masterconn *eptr) {
 	uint64_t tdusedspace,tdtotalspace;
 	uint32_t chunkcount,tdchunkcount;
 
-	myip = csserv_getlistenip();
-	myport = csserv_getlistenport();
+	myip = mainNetworkThreadGetListenIp();
+	myport = mainNetworkThreadGetListenPort();
 	masterconn_create_attached_packet(eptr, cstoma::registerHost::build(myip, myport, Timeout_ms, LIZARDFS_VERSHEX));
+
+	std::vector<ChunkWithVersionAndType> chunks;
+	std::vector<ChunkWithType> recheck_list;
+
 	hdd_get_chunks_begin();
-	std::vector<ChunkWithVersion> chunks;
-	hdd_get_chunks_next_list_data(chunks);
+	hdd_get_chunks_next_list_data(chunks, recheck_list);
 	while (!chunks.empty()) {
 		masterconn_create_attached_packet(eptr, cstoma::registerChunks::build(chunks));
-		chunks.resize(0);
-		hdd_get_chunks_next_list_data(chunks);
+		hdd_get_chunks_next_list_data(chunks, recheck_list);
 	}
 	hdd_get_chunks_end();
+
+	hdd_get_chunks_next_list_data_recheck(chunks, recheck_list);
+	while (!chunks.empty()) {
+		masterconn_create_attached_packet(eptr, cstoma::registerChunks::build(chunks));
+		hdd_get_chunks_next_list_data_recheck(chunks, recheck_list);
+	}
+
 	hdd_get_space(&usedspace,&totalspace,&chunkcount,&tdusedspace,&tdtotalspace,&tdchunkcount);
 	auto registerSpace = cstoma::registerSpace::build(
 			usedspace, totalspace, chunkcount, tdusedspace, tdtotalspace, tdchunkcount);
@@ -193,7 +189,7 @@ void masterconn_sendregister(masterconn *eptr) {
 void masterconn_check_hdd_reports() {
 	masterconn *eptr = masterconnsingleton;
 	uint32_t errorcounter;
-	if (eptr->mode==DATA || eptr->mode==HEADER) {
+	if (eptr->mode == CONNECTED) {
 		if (hdd_spacechanged()) {
 			uint64_t usedspace,totalspace,tdusedspace,tdtotalspace;
 			uint32_t chunkcount,tdchunkcount;
@@ -209,30 +205,29 @@ void masterconn_check_hdd_reports() {
 			errorcounter--;
 		}
 
-		MooseFSVector<uint64_t> chunks;
-		hdd_get_damaged_chunks(chunks);
-		if (!chunks.empty()) {
-			masterconn_create_attached_moosefs_packet(eptr, CSTOMA_CHUNK_DAMAGED, chunks);
+		std::vector<ChunkWithType> chunks_with_type;
+		hdd_get_damaged_chunks(chunks_with_type, 1000);
+		if (!chunks_with_type.empty()) {
+			masterconn_create_attached_packet(eptr, cstoma::chunkDamaged::build(chunks_with_type));
 		}
 
-		chunks.clear();
-		hdd_get_lost_chunks(chunks, LOSTCHUNKLIMIT);
-		if (!chunks.empty()) {
-			masterconn_create_attached_moosefs_packet(eptr, CSTOMA_CHUNK_LOST, chunks);
+		hdd_get_lost_chunks(chunks_with_type, 1000);
+		if (!chunks_with_type.empty()) {
+			masterconn_create_attached_packet(eptr, cstoma::chunkLost::build(chunks_with_type));
 		}
 
-		MooseFSVector<ChunkWithVersion> chunksWithVersions;
-		hdd_get_new_chunks(chunksWithVersions, NEWCHUNKLIMIT);
-		if (!chunksWithVersions.empty()) {
-			masterconn_create_attached_moosefs_packet(eptr, CSTOMA_CHUNK_NEW,
-					chunksWithVersions);
+		std::vector<ChunkWithVersionAndType> chunks_with_version;
+		hdd_get_new_chunks(chunks_with_version, 1000);
+		if (!chunks_with_version.empty()) {
+			masterconn_create_attached_packet(eptr, cstoma::chunkNew::build(chunks_with_version));
 		}
 	}
 }
-void masterconn_jobfinished(uint8_t status,void *packet) {
+
+void masterconn_jobfinished(uint8_t status, void *packet) {
 	uint8_t *ptr;
 	masterconn *eptr = masterconnsingleton;
-	if (eptr->mode==DATA || eptr->mode==HEADER) {
+	if (eptr->mode == CONNECTED) {
 		ptr = masterconn_get_packet_data(packet);
 		ptr[8]=status;
 		masterconn_attach_packet(eptr,packet);
@@ -241,10 +236,21 @@ void masterconn_jobfinished(uint8_t status,void *packet) {
 	}
 }
 
+void masterconn_lizjobfinished(uint8_t status, void *packet) {
+	OutputPacket* outputPacket = static_cast<OutputPacket*>(packet);
+	masterconn *eptr = masterconnsingleton;
+	if (eptr->mode == CONNECTED) {
+		cstoma::overwriteStatusField(outputPacket->packet, status);
+		masterconn_attach_packet(eptr, packet);
+	} else {
+		masterconn_delete_packet(packet);
+	}
+}
+
 void masterconn_chunkopfinished(uint8_t status,void *packet) {
 	uint8_t *ptr;
 	masterconn *eptr = masterconnsingleton;
-	if (eptr->mode==DATA || eptr->mode==HEADER) {
+	if (eptr->mode == CONNECTED) {
 		ptr = masterconn_get_packet_data(packet);
 		ptr[32]=status;
 		masterconn_attach_packet(eptr,packet);
@@ -257,7 +263,7 @@ void masterconn_replicationfinished(uint8_t status,void *packet) {
 	uint8_t *ptr;
 	masterconn *eptr = masterconnsingleton;
 //      syslog(LOG_NOTICE,"job replication status: %" PRIu8,status);
-	if (eptr->mode==DATA || eptr->mode==HEADER) {
+	if (eptr->mode == CONNECTED) {
 		ptr = masterconn_get_packet_data(packet);
 		ptr[12]=status;
 		masterconn_attach_packet(eptr,packet);
@@ -272,134 +278,80 @@ void masterconn_unwantedjobfinished(uint8_t status,void *packet) {
 }
 
 
-void masterconn_create(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
-	uint32_t version;
-	uint8_t *ptr;
-	void *packet;
+void masterconn_create(masterconn */*eptr*/, const std::vector<uint8_t> &data) {
+	uint64_t chunkId;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
+	uint32_t chunkVersion;
 
-	if (length!=8+4) {
-		syslog(LOG_NOTICE,"MATOCS_CREATE - wrong size (%" PRIu32 "/12)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	chunkid = get64bit(&data);
-	version = get32bit(&data);
-	packet = masterconn_create_detached_packet(CSTOMA_CREATE,8+1);
-	ptr = masterconn_get_packet_data(packet);
-	put64bit(&ptr,chunkid);
-	job_create(jpool,masterconn_jobfinished,packet,chunkid,version);
+	matocs::createChunk::deserialize(data, chunkId, chunkType, chunkVersion);
+	OutputPacket *outputPacket = new OutputPacket;
+	cstoma::createChunk::serialize(outputPacket->packet, chunkId, chunkType, LIZARDFS_STATUS_OK);
+	job_create(jpool, masterconn_lizjobfinished, outputPacket, chunkId, chunkType, chunkVersion);
 }
 
-void masterconn_delete(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
-	uint32_t version;
-	uint8_t *ptr;
-	void *packet;
+void masterconn_delete(masterconn */*eptr*/, const std::vector<uint8_t>& data) {
+	uint64_t chunkId;
+	uint32_t chunkVersion;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
 
-	if (length!=8+4) {
-		syslog(LOG_NOTICE,"MATOCS_DELETE - wrong size (%" PRIu32 "/12)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	chunkid = get64bit(&data);
-	version = get32bit(&data);
-	packet = masterconn_create_detached_packet(CSTOMA_DELETE,8+1);
-	ptr = masterconn_get_packet_data(packet);
-	put64bit(&ptr,chunkid);
-	job_delete(jpool,masterconn_jobfinished,packet,chunkid,version);
+	matocs::deleteChunk::deserialize(data, chunkId, chunkType, chunkVersion);
+	OutputPacket* outputPacket = new OutputPacket;
+	cstoma::deleteChunk::serialize(outputPacket->packet, chunkId, chunkType, 0);
+	job_delete(jpool, masterconn_lizjobfinished, outputPacket, chunkId, chunkVersion, chunkType);
 }
 
-void masterconn_setversion(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
-	uint32_t version;
-	uint32_t newversion;
-	uint8_t *ptr;
-	void *packet;
+void masterconn_setversion(masterconn */*eptr*/, const std::vector<uint8_t>& data) {
+	uint64_t chunkId;
+	uint32_t chunkVersion;
+	uint32_t newVersion;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
 
-	if (length!=8+4+4) {
-		syslog(LOG_NOTICE,"MATOCS_SET_VERSION - wrong size (%" PRIu32 "/16)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	chunkid = get64bit(&data);
-	newversion = get32bit(&data);
-	version = get32bit(&data);
-	packet = masterconn_create_detached_packet(CSTOMA_SET_VERSION,8+1);
-	ptr = masterconn_get_packet_data(packet);
-	put64bit(&ptr,chunkid);
-	job_version(jpool,masterconn_jobfinished,packet,chunkid,version,newversion);
+	matocs::setVersion::deserialize(data, chunkId, chunkType, chunkVersion, newVersion);
+	OutputPacket* outputPacket = new OutputPacket;
+	cstoma::setVersion::serialize(outputPacket->packet, chunkId, chunkType, 0);
+	job_version(jpool, masterconn_lizjobfinished, outputPacket, chunkId, chunkVersion, chunkType,
+			newVersion);
 }
 
-void masterconn_duplicate(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
-	uint32_t version;
-	uint64_t copychunkid;
-	uint32_t copyversion;
-	uint8_t *ptr;
-	void *packet;
+void masterconn_duplicate(masterconn* /*eptr*/,const std::vector<uint8_t>& data) {
+	uint64_t newChunkId, oldChunkId;
+	uint32_t newChunkVersion, oldChunkVersion;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
 
-	if (length!=8+4+8+4) {
-		syslog(LOG_NOTICE,"MATOCS_DUPLICATE - wrong size (%" PRIu32 "/24)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	copychunkid = get64bit(&data);
-	copyversion = get32bit(&data);
-	chunkid = get64bit(&data);
-	version = get32bit(&data);
-	packet = masterconn_create_detached_packet(CSTOMA_DUPLICATE,8+1);
-	ptr = masterconn_get_packet_data(packet);
-	put64bit(&ptr,copychunkid);
-	job_duplicate(jpool,masterconn_jobfinished,packet,chunkid,version,version,copychunkid,copyversion);
+	matocs::duplicateChunk::deserialize(data, newChunkId, newChunkVersion, chunkType,
+			oldChunkId, oldChunkVersion);
+	OutputPacket* outputPacket = new OutputPacket;
+	cstoma::duplicateChunk::serialize(outputPacket->packet, newChunkId, chunkType, 0);
+	job_duplicate(jpool, masterconn_lizjobfinished, outputPacket, oldChunkId,
+			oldChunkVersion, oldChunkVersion, chunkType, newChunkId, newChunkVersion);
 }
 
-void masterconn_truncate(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
+void masterconn_truncate(masterconn */*eptr*/, const std::vector<uint8_t>& data) {
+	uint64_t chunkId;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
 	uint32_t version;
-	uint32_t leng;
-	uint32_t newversion;
-	uint8_t *ptr;
-	void *packet;
+	uint32_t chunkLength;
+	uint32_t newVersion;
 
-	if (length!=8+4+4+4) {
-		syslog(LOG_NOTICE,"MATOCS_TRUNCATE - wrong size (%" PRIu32 "/20)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	chunkid = get64bit(&data);
-	leng = get32bit(&data);
-	newversion = get32bit(&data);
-	version = get32bit(&data);
-	packet = masterconn_create_detached_packet(CSTOMA_TRUNCATE,8+1);
-	ptr = masterconn_get_packet_data(packet);
-	put64bit(&ptr,chunkid);
-	job_truncate(jpool,masterconn_jobfinished,packet,chunkid,version,newversion,leng);
+	matocs::truncateChunk::deserialize(data, chunkId, chunkType, chunkLength, newVersion, version);
+	OutputPacket* outputPacket = new OutputPacket;
+	cstoma::truncate::serialize(outputPacket->packet, chunkId, chunkType, 0);
+	job_truncate(jpool, masterconn_lizjobfinished, outputPacket, chunkId, chunkType, version,
+			newVersion, chunkLength);
 }
 
-void masterconn_duptrunc(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
-	uint32_t version;
-	uint64_t copychunkid;
-	uint32_t copyversion;
-	uint32_t leng;
-	uint8_t *ptr;
-	void *packet;
+void masterconn_duptrunc(masterconn* /*eptr*/, const std::vector<uint8_t>& data) {
+	uint64_t chunkId, copyChunkId;
+	uint32_t chunkVersion, copyChunkVersion;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
+	uint32_t newLength;
 
-	if (length!=8+4+8+4+4) {
-		syslog(LOG_NOTICE,"MATOCS_DUPTRUNC - wrong size (%" PRIu32 "/28)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	copychunkid = get64bit(&data);
-	copyversion = get32bit(&data);
-	chunkid = get64bit(&data);
-	version = get32bit(&data);
-	leng = get32bit(&data);
-	packet = masterconn_create_detached_packet(CSTOMA_DUPTRUNC,8+1);
-	ptr = masterconn_get_packet_data(packet);
-	put64bit(&ptr,copychunkid);
-	job_duptrunc(jpool,masterconn_jobfinished,packet,chunkid,version,version,copychunkid,copyversion,leng);
+	matocs::duptruncChunk::deserialize(data, copyChunkId, copyChunkVersion,
+			chunkType, chunkId, chunkVersion, newLength);
+	OutputPacket* outputPacket = new OutputPacket;
+	cstoma::duptruncChunk::serialize(outputPacket->packet, copyChunkId, chunkType, 0);
+	job_duptrunc(jpool, masterconn_lizjobfinished, outputPacket, chunkId,
+			chunkVersion, chunkVersion, chunkType, copyChunkId, copyChunkVersion, newLength);
 }
 
 void masterconn_chunkop(masterconn *eptr,const uint8_t *data,uint32_t length) {
@@ -430,10 +382,36 @@ void masterconn_chunkop(masterconn *eptr,const uint8_t *data,uint32_t length) {
 	put64bit(&ptr,copychunkid);
 	put32bit(&ptr,copyversion);
 	put32bit(&ptr,leng);
-	job_chunkop(jpool,masterconn_chunkopfinished,packet,chunkid,version,newversion,copychunkid,copyversion,leng);
+	job_chunkop(jpool, masterconn_chunkopfinished, packet, chunkid, version,
+			slice_traits::standard::ChunkPartType(), newversion, copychunkid, copyversion, leng);
 }
 
-void masterconn_replicate(masterconn *eptr,const uint8_t *data,uint32_t length) {
+void masterconn_replicate(const std::vector<uint8_t>& data) {
+	uint64_t chunkId;
+	ChunkPartType chunkType = slice_traits::standard::ChunkPartType();
+	uint32_t chunkVersion;
+	uint32_t sourcesBufferSize;
+	const uint8_t* sourcesBuffer;
+
+	matocs::replicateChunk::deserializePartial(data,
+			chunkId, chunkVersion, chunkType, sourcesBuffer);
+	sourcesBufferSize = data.size() - (sourcesBuffer - data.data());
+
+	OutputPacket* outputPacket = new OutputPacket;
+	cstoma::replicateChunk::serialize(outputPacket->packet,
+			chunkId, chunkType, LIZARDFS_STATUS_OK, chunkVersion);
+	DEBUG_LOG("cs.matocs.replicate") << chunkId;
+	if (hdd_scans_in_progress()) {
+		// Folder scan in progress - replication is not possible
+		masterconn_lizjobfinished(LIZARDFS_ERROR_WAITING, outputPacket);
+	} else {
+		job_replicate(jpool, masterconn_lizjobfinished, outputPacket,
+				chunkId, chunkVersion, chunkType, sourcesBufferSize, sourcesBuffer);
+	}
+
+}
+
+void masterconn_legacy_replicate(masterconn *eptr,const uint8_t *data,uint32_t length) {
 	uint64_t chunkid;
 	uint32_t version;
 	uint32_t ip;
@@ -453,13 +431,16 @@ void masterconn_replicate(masterconn *eptr,const uint8_t *data,uint32_t length)
 	ptr = masterconn_get_packet_data(packet);
 	put64bit(&ptr,chunkid);
 	put32bit(&ptr,version);
-	if (length==8+4+4+2) {
+
+	if (hdd_scans_in_progress()) {
+		masterconn_replicationfinished(LIZARDFS_ERROR_WAITING, packet);
+	} else if (length==8+4+4+2) {
 		ip = get32bit(&data);
 		port = get16bit(&data);
 //              syslog(LOG_NOTICE,"start job replication (%08" PRIX64 ":%04" PRIX32 ":%04" PRIX32 ":%02" PRIX16 ")",chunkid,version,ip,port);
-		job_replicate_simple(jpool,masterconn_replicationfinished,packet,chunkid,version,ip,port);
+		job_legacy_replicate_simple(jpool,masterconn_replicationfinished,packet,chunkid,version,ip,port);
 	} else {
-		job_replicate(jpool,masterconn_replicationfinished,packet,chunkid,version,(length-12)/18,data);
+		job_legacy_replicate(jpool,masterconn_replicationfinished,packet,chunkid,version,(length-12)/18,data);
 	}
 }
 
@@ -531,74 +512,57 @@ void masterconn_structure_log_rotate(masterconn *eptr,const uint8_t *data,uint32
 }
 */
 
-void masterconn_chunk_checksum(masterconn *eptr,const uint8_t *data,uint32_t length) {
-	uint64_t chunkid;
-	uint32_t version;
-	uint8_t status;
-	uint32_t checksum;
-
-	if (length!=8+4) {
-		syslog(LOG_NOTICE,"ANTOCS_CHUNK_CHECKSUM - wrong size (%" PRIu32 "/12)",length);
-		eptr->mode = KILL;
-		return;
-	}
-	chunkid = get64bit(&data);
-	version = get32bit(&data);
-	status = hdd_get_checksum(chunkid,version,&checksum);
-	if (status!=STATUS_OK) {
-		masterconn_create_attached_moosefs_packet(
-				eptr, CSTOAN_CHUNK_CHECKSUM, chunkid, version, status);
-	} else {
-		masterconn_create_attached_moosefs_packet(
-				eptr, CSTOAN_CHUNK_CHECKSUM, chunkid, version, checksum);
-	}
-}
-
-void masterconn_gotpacket(masterconn *eptr,uint32_t type,const uint8_t *data,uint32_t length) {
-	switch (type) {
+void masterconn_gotpacket(masterconn *eptr, PacketHeader header, const MessageBuffer& message) try {
+	switch (header.type) {
 		case ANTOAN_NOP:
 			break;
 		case ANTOAN_UNKNOWN_COMMAND: // for future use
 			break;
 		case ANTOAN_BAD_COMMAND_SIZE: // for future use
 			break;
-		case MATOCS_CREATE:
-			masterconn_create(eptr,data,length);
+		case LIZ_MATOCS_CREATE_CHUNK:
+			masterconn_create(eptr, message);
 			break;
-		case MATOCS_DELETE:
-			masterconn_delete(eptr,data,length);
+		case LIZ_MATOCS_DELETE_CHUNK:
+			masterconn_delete(eptr, message);
 			break;
-		case MATOCS_SET_VERSION:
-			masterconn_setversion(eptr,data,length);
+		case LIZ_MATOCS_SET_VERSION:
+			masterconn_setversion(eptr, message);
 			break;
-		case MATOCS_DUPLICATE:
-			masterconn_duplicate(eptr,data,length);
+		case LIZ_MATOCS_DUPLICATE_CHUNK:
+			masterconn_duplicate(eptr, message);
 			break;
 		case MATOCS_REPLICATE:
-			masterconn_replicate(eptr,data,length);
+			masterconn_legacy_replicate(eptr, message.data(), message.size());
+			break;
+		case LIZ_MATOCS_REPLICATE_CHUNK:
+			masterconn_replicate(message);
 			break;
 		case MATOCS_CHUNKOP:
-			masterconn_chunkop(eptr,data,length);
+			masterconn_chunkop(eptr, message.data(), message.size());
 			break;
-		case MATOCS_TRUNCATE:
-			masterconn_truncate(eptr,data,length);
+		case LIZ_MATOCS_TRUNCATE:
+			masterconn_truncate(eptr, message);
 			break;
-		case MATOCS_DUPTRUNC:
-			masterconn_duptrunc(eptr,data,length);
+		case LIZ_MATOCS_DUPTRUNC_CHUNK:
+			masterconn_duptrunc(eptr, message);
 			break;
 //              case MATOCS_STRUCTURE_LOG:
-//                      masterconn_structure_log(eptr,data,length);
+//                      masterconn_structure_log(eptr, message.data(), message.size());
 //                      break;
 //              case MATOCS_STRUCTURE_LOG_ROTATE:
-//                      masterconn_structure_log_rotate(eptr,data,length);
+//                      masterconn_structure_log_rotate(eptr, message.data(), message.size());
 //                      break;
-		case ANTOCS_CHUNK_CHECKSUM:
-			masterconn_chunk_checksum(eptr,data,length);
-			break;
 		default:
-			syslog(LOG_NOTICE,"got unknown message (type:%" PRIu32 ")",type);
+			syslog(LOG_NOTICE,"got unknown message (type:%" PRIu32 ")", header.type);
 			eptr->mode = KILL;
 	}
+} catch (IncorrectDeserializationException& e) {
+	syslog(LOG_NOTICE,
+			"chunkserver <-> master module: got inconsistent message "
+			"(type:%" PRIu32 ", length:%" PRIu32"), %s",
+			header.type, uint32_t(message.size()), e.what());
+	eptr->mode = KILL;
 }
 
 
@@ -610,7 +574,7 @@ void masterconn_term(void) {
 
 	if (eptr->mode!=FREE && eptr->mode!=CONNECTING) {
 		tcpclose(eptr->sock);
-		eptr->inputpacket.packet.clear();
+		eptr->inputPacket.reset();
 	}
 
 	delete eptr;
@@ -623,10 +587,8 @@ void masterconn_term(void) {
 
 void masterconn_connected(masterconn *eptr) {
 	tcpnodelay(eptr->sock);
-	eptr->mode=HEADER;
-	eptr->inputpacket.bytesLeft = PacketHeader::kSize;
-	eptr->inputpacket.startptr = eptr->hdrbuff;
-	eptr->inputpacket.packet.clear();
+	eptr->mode = CONNECTED;
+	eptr->inputPacket.reset();
 
 	masterconn_sendregister(eptr);
 	eptr->lastread.reset();
@@ -710,68 +672,58 @@ void masterconn_connecttest(masterconn *eptr) {
 }
 
 void masterconn_read(masterconn *eptr) {
-	int32_t i;
-	for (;;) {
-		if (job_pool_jobs_count(jpool)>=(BGJOBSCNT*9)/10) {
+	ActiveLoopWatchdog watchdog(std::chrono::milliseconds(20));
+
+	watchdog.start();
+	while (eptr->mode != KILL) {
+		if (job_pool_jobs_count(jpool) >= (BGJOBSCNT * 9) / 10) {
 			return;
 		}
-		i=read(eptr->sock,eptr->inputpacket.startptr,eptr->inputpacket.bytesLeft);
-		if (i==0) {
-			syslog(LOG_NOTICE,"connection reset by Master");
+		uint32_t bytesToRead = eptr->inputPacket.bytesToBeRead();
+		ssize_t ret = read(eptr->sock, eptr->inputPacket.pointerToBeReadInto(), bytesToRead);
+		if (ret == 0) {
+			lzfs_silent_syslog(LOG_NOTICE, "connection reset by Master");
 			eptr->mode = KILL;
 			return;
-		}
-		if (i<0) {
-			if (errno!=EAGAIN) {
-				lzfs_silent_errlog(LOG_NOTICE,"read from Master error");
+		} else if (ret < 0) {
+			if (errno != EAGAIN) {
+				lzfs_silent_errlog(LOG_NOTICE, "read from Master error");
 				eptr->mode = KILL;
 			}
 			return;
 		}
-		stats_bytesin+=i;
-		eptr->inputpacket.startptr+=i;
-		eptr->inputpacket.bytesLeft-=i;
 
-		if (eptr->inputpacket.bytesLeft>0) {
+		stats_bytesin += ret;
+		try {
+			eptr->inputPacket.increaseBytesRead(ret);
+		} catch (InputPacketTooLongException& ex) {
+			lzfs_silent_syslog(LOG_WARNING, "reading from master: %s", ex.what());
+			eptr->mode = KILL;
+			return;
+		}
+		if (ret == bytesToRead && !eptr->inputPacket.hasData()) {
+			// there might be more data to read in socket's buffer
+			continue;
+		} else if (!eptr->inputPacket.hasData()) {
 			return;
 		}
 
-		PacketHeader header;
-		deserializePacketHeader(eptr->hdrbuff, PacketHeader::kSize, header);
-		if (eptr->mode==HEADER) {
-			if (header.length > 0) {
-				if (header.length > MaxPacketSize) {
-					syslog(LOG_WARNING, "Master packet too long (%" PRIu32 "/%u)",
-							header.length, MaxPacketSize);
-					eptr->mode = KILL;
-					return;
-				}
-				eptr->inputpacket.packet.resize(header.length);
-				eptr->inputpacket.bytesLeft = header.length;
-				eptr->inputpacket.startptr = eptr->inputpacket.packet.data();
-				eptr->mode = DATA;
-				continue;
-			}
-			eptr->mode = DATA;
-		}
+		masterconn_gotpacket(eptr, eptr->inputPacket.getHeader(), eptr->inputPacket.getData());
+		eptr->inputPacket.reset();
 
-		if (eptr->mode==DATA) {
-
-			eptr->mode=HEADER;
-			eptr->inputpacket.bytesLeft = PacketHeader::kSize;
-			eptr->inputpacket.startptr = eptr->hdrbuff;
-
-			masterconn_gotpacket(eptr, header.type, eptr->inputpacket.packet.data(), header.length);
-
-			eptr->inputpacket.packet.clear();
+		if (watchdog.expired()) {
+			break;
 		}
 	}
 }
 
 void masterconn_write(masterconn *eptr) {
+	ActiveLoopWatchdog watchdog(std::chrono::milliseconds(20));
 	int32_t i;
+
+	watchdog.start();
 	while (!eptr->outputPackets.empty()) {
-		auto& pack = eptr->outputPackets.front();
+		OutputPacket& pack = eptr->outputPackets.front();
 		i=write(eptr->sock, pack.packet.data() + pack.bytesSent,
 				pack.packet.size() - pack.bytesSent);
 		if (i<0) {
@@ -787,12 +739,16 @@ void masterconn_write(masterconn *eptr) {
 			return;
 		}
 		eptr->outputPackets.pop_front();
+
+		if (watchdog.expired()) {
+			break;
+		}
 	}
 }
 
 
-void masterconn_desc(struct pollfd *pdesc,uint32_t *ndesc) {
-	uint32_t pos = *ndesc;
+void masterconn_desc(std::vector<pollfd> &pdesc) {
+	LOG_AVG_TILL_END_OF_SCOPE0("master_desc");
 	masterconn *eptr = masterconnsingleton;
 
 	eptr->pdescpos = -1;
@@ -801,33 +757,27 @@ void masterconn_desc(struct pollfd *pdesc,uint32_t *ndesc) {
 	if (eptr->mode==FREE || eptr->sock<0) {
 		return;
 	}
-	if (eptr->mode==HEADER || eptr->mode==DATA) {
-		pdesc[pos].fd = jobfd;
-		pdesc[pos].events = POLLIN;
-		jobfdpdescpos = pos;
-		pos++;
+	if (eptr->mode == CONNECTED) {
+		pdesc.push_back({jobfd,POLLIN,0});
+		jobfdpdescpos = pdesc.size() - 1;
 		if (job_pool_jobs_count(jpool)<(BGJOBSCNT*9)/10) {
-			pdesc[pos].fd = eptr->sock;
-			pdesc[pos].events = POLLIN;
-			eptr->pdescpos = pos;
-			pos++;
+			pdesc.push_back({eptr->sock,POLLIN,0});
+			eptr->pdescpos = pdesc.size() - 1;
 		}
 	}
-	if (((eptr->mode==HEADER || eptr->mode==DATA) && !eptr->outputPackets.empty())
+	if (((eptr->mode == CONNECTED) && !eptr->outputPackets.empty())
 			|| eptr->mode==CONNECTING) {
 		if (eptr->pdescpos>=0) {
 			pdesc[eptr->pdescpos].events |= POLLOUT;
 		} else {
-			pdesc[pos].fd = eptr->sock;
-			pdesc[pos].events = POLLOUT;
-			eptr->pdescpos = pos;
-			pos++;
+			pdesc.push_back({eptr->sock,POLLOUT,0});
+			eptr->pdescpos = pdesc.size() - 1;
 		}
 	}
-	*ndesc = pos;
 }
 
-void masterconn_serve(struct pollfd *pdesc) {
+void masterconn_serve(const std::vector<pollfd> &pdesc) {
+	LOG_AVG_TILL_END_OF_SCOPE0("master_serve");
 	masterconn *eptr = masterconnsingleton;
 
 	if (eptr->pdescpos>=0 && (pdesc[eptr->pdescpos].revents & (POLLHUP | POLLERR))) {
@@ -842,27 +792,27 @@ void masterconn_serve(struct pollfd *pdesc) {
 			masterconn_connecttest(eptr);
 		}
 	} else {
-		if ((eptr->mode==HEADER || eptr->mode==DATA) && jobfdpdescpos>=0 && (pdesc[jobfdpdescpos].revents & POLLIN)) { // FD_ISSET(jobfd,rset)) {
+		if ((eptr->mode == CONNECTED) && jobfdpdescpos>=0 && (pdesc[jobfdpdescpos].revents & POLLIN)) { // FD_ISSET(jobfd,rset)) {
 			job_pool_check_jobs(jpool);
 		}
 		if (eptr->pdescpos>=0) {
-			if ((eptr->mode==HEADER || eptr->mode==DATA) && (pdesc[eptr->pdescpos].revents & POLLIN)) { // FD_ISSET(eptr->sock,rset)) {
+			if ((eptr->mode == CONNECTED) && (pdesc[eptr->pdescpos].revents & POLLIN)) { // FD_ISSET(eptr->sock,rset)) {
 				eptr->lastread.reset();
 				masterconn_read(eptr);
 			}
-			if ((eptr->mode==HEADER || eptr->mode==DATA) && (pdesc[eptr->pdescpos].revents & POLLOUT)) { // FD_ISSET(eptr->sock,wset)) {
+			if ((eptr->mode == CONNECTED) && (pdesc[eptr->pdescpos].revents & POLLOUT)) { // FD_ISSET(eptr->sock,wset)) {
 				eptr->lastwrite.reset();
 				masterconn_write(eptr);
 			}
-			if ((eptr->mode==HEADER || eptr->mode==DATA) && eptr->lastread.elapsed_ms() > Timeout_ms) {
+			if ((eptr->mode == CONNECTED) && eptr->lastread.elapsed_ms() > Timeout_ms) {
 				eptr->mode = KILL;
 			}
-			if ((eptr->mode==HEADER || eptr->mode==DATA) && eptr->lastwrite.elapsed_ms() > (Timeout_ms/3) && eptr->outputPackets.empty()) {
+			if ((eptr->mode == CONNECTED) && eptr->lastwrite.elapsed_ms() > (Timeout_ms/3) && eptr->outputPackets.empty()) {
 				masterconn_create_attached_moosefs_packet(eptr, ANTOAN_NOP);
 			}
 		}
 	}
-	if (eptr->mode==HEADER || eptr->mode==DATA) {
+	if (eptr->mode == CONNECTED) {
 		uint32_t jobscnt = job_pool_jobs_count(jpool);
 		if (jobscnt>=stats_maxjobscnt) {
 			stats_maxjobscnt=jobscnt;
@@ -871,7 +821,7 @@ void masterconn_serve(struct pollfd *pdesc) {
 	if (eptr->mode == KILL) {
 		job_pool_disable_and_change_callback_all(jpool,masterconn_unwantedjobfinished);
 		tcpclose(eptr->sock);
-		eptr->inputpacket.packet.clear();
+		eptr->inputPacket.reset();
 		eptr->outputPackets.clear();
 		eptr->mode = FREE;
 	}
@@ -891,8 +841,8 @@ static uint32_t get_cfg_timeout() {
 /// Read the label from configuration file and return true if it's changed to a valid one
 bool masterconn_load_label() {
 	std::string oldLabel = gLabel;
-	gLabel = cfg_getstring("LABEL", kMediaLabelWildcard);
-	if (!isMediaLabelValid(gLabel)) {
+	gLabel = cfg_getstring("LABEL", MediaLabelManager::kWildcard);
+	if (!MediaLabelManager::isLabelValid(gLabel)) {
 		lzfs_pretty_syslog(LOG_WARNING,"invalid label '%s'", gLabel.c_str());
 		return false;
 	}
@@ -975,15 +925,18 @@ int masterconn_init(void) {
 		return -1;
 	}
 
-	jpool = job_pool_new(10,BGJOBSCNT,&jobfd);
-	if (jpool==NULL) {
-		return -1;
-	}
-
 	main_eachloopregister(masterconn_check_hdd_reports);
-	reconnect_hook = main_timeregister(TIMEMODE_RUN_LATE,ReconnectionDelay,rndu32_ranged(ReconnectionDelay),masterconn_reconnect);
+	reconnect_hook = main_timeregister(TIMEMODE_RUN_LATE,ReconnectionDelay,rnd_ranged<uint32_t>(ReconnectionDelay),masterconn_reconnect);
 	main_destructregister(masterconn_term);
 	main_pollregister(masterconn_desc,masterconn_serve);
 	main_reloadregister(masterconn_reload);
 	return 0;
 }
+
+int masterconn_init_threads(void) {
+	jpool = job_pool_new(10,BGJOBSCNT,&jobfd);
+	if (jpool==NULL) {
+		return -1;
+	}
+	return 0;
+}
