@@ -1,5 +1,5 @@
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -19,10 +19,6 @@
 #include "common/platform.h"
 #include "init.h"
 
-#ifndef MFSMAXFILES
-#define MFSMAXFILES 5000
-#endif
-
 #include <errno.h>
 #include <fcntl.h>
 #include <grp.h>
@@ -32,15 +28,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
-#include <syslog.h>
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <syslog.h>
 #include <time.h>
 #include <unistd.h>
 #include <algorithm>
+#include <atomic>
 #include <list>
 #include <memory>
 
@@ -52,9 +49,10 @@
 #include "common/main.h"
 #include "common/massert.h"
 #include "common/mfserr.h"
-#include "common/MFSCommunication.h"
 #include "common/setup.h"
 #include "common/slogger.h"
+#include "common/time_utils.h"
+#include "protocol/MFSCommunication.h"
 
 #if defined(LIZARDFS_HAVE_MLOCKALL)
 #  if defined(LIZARDFS_HAVE_SYS_MMAN_H)
@@ -68,6 +66,19 @@
 #  endif
 #endif
 
+#if defined(LIZARDFS_HAVE_PAM)
+#  include <security/pam_appl.h>
+#  include <security/pam_misc.h>
+#endif
+
+#ifndef LIZARDFS_MAX_FILES
+#  define LIZARDFS_MAX_FILES 5000
+#endif
+
+#if defined(LIZARDFS_HAVE_SYSTEMD_SD_DAEMON_H)
+#include <systemd/sd-daemon.h>
+#endif
+
 #define STR_AUX(x) #x
 #define STR(x) STR_AUX(x)
 
@@ -85,7 +96,7 @@ enum class ExitingStatus {
 	kRunning = 0,
 	kWantExit = 1,
 	kCanExit = 2,
-	kDoExit = 3,
+	kDoExit = 3
 };
 
 static bool nextPollNonblocking = false;
@@ -121,15 +132,14 @@ typedef struct rlentry {
 
 static rlentry *rlhead=NULL;
 
-
 typedef struct pollentry {
-	void (*desc)(struct pollfd *,uint32_t *);
-	void (*serve)(struct pollfd *);
-	struct pollentry *next;
+	void (*desc)(std::vector<pollfd>&);
+	void (*serve)(const std::vector<pollfd>&);
 } pollentry;
 
-static pollentry *pollhead=NULL;
-
+namespace {
+std::list<pollentry> gPollEntries;
+}
 
 typedef struct eloopentry {
 	void (*fun)(void);
@@ -138,17 +148,21 @@ typedef struct eloopentry {
 
 static eloopentry *eloophead=NULL;
 
+#if defined(LIZARDFS_HAVE_PAM)
+static pam_handle_t *gPAMHandle=NULL;
+#endif
 
 struct timeentry {
 	typedef void (*fun_t)(void);
-	timeentry(uint32_t ne, uint32_t sec, uint32_t off, int mod, fun_t f)
-		: nextevent(ne), seconds(sec), offset(off), mode(mod), fun(f) {
+	timeentry(uint64_t ne, uint64_t sec, uint64_t off, int mod, fun_t f, bool ms)
+		: nextevent(ne), period(sec), offset(off), mode(mod), fun(f), millisecond_precision(ms) {
 	}
-	uint32_t nextevent;
-	uint32_t seconds;
-	uint32_t offset;
-	int mode;
-	fun_t fun;
+	uint64_t nextevent;
+	uint64_t period;
+	uint64_t offset;
+	int      mode;
+	fun_t    fun;
+	bool     millisecond_precision;
 };
 
 typedef std::list<timeentry> TimeEntries;
@@ -156,8 +170,8 @@ namespace {
 TimeEntries gTimeEntries;
 }
 
-static uint32_t now;
-static uint64_t usecnow;
+static std::atomic<uint32_t> now;
+static std::atomic<uint64_t> usecnow;
 static bool gRunAsDaemon = true;
 static std::vector<std::string> gExtraArguments;
 static ExitingStatus gExitingStatus = ExitingStatus::kRunning;
@@ -224,13 +238,8 @@ void main_reloadregister (void (*fun)(void)) {
 	rlhead = aux;
 }
 
-void main_pollregister (void (*desc)(struct pollfd *,uint32_t *),void (*serve)(struct pollfd *)) {
-	pollentry *aux=(pollentry*)malloc(sizeof(pollentry));
-	passert(aux);
-	aux->desc = desc;
-	aux->serve = serve;
-	aux->next = pollhead;
-	pollhead = aux;
+void main_pollregister (void (*desc)(std::vector<pollfd>&),void (*serve)(const std::vector<pollfd>&)) {
+	gPollEntries.push_back({desc,serve});
 }
 
 void main_eachloopregister (void (*fun)(void)) {
@@ -241,44 +250,60 @@ void main_eachloopregister (void (*fun)(void)) {
 	eloophead = aux;
 }
 
-void* main_timeregister (int mode,uint32_t seconds,uint32_t offset,void (*fun)(void)) {
-	if (seconds==0 || offset>=seconds) {
+void *main_timeregister(int mode, uint64_t seconds, uint64_t offset, void (*fun)(void)) {
+	if (seconds == 0 || offset >= seconds) {
 		return NULL;
 	}
-	uint32_t nextevent = ((now / seconds) * seconds) + offset;
-	while (nextevent<now) {
-		nextevent+=seconds;
+
+	uint64_t nextevent = ((now + seconds) / seconds) * seconds + offset;
+
+	gTimeEntries.push_front(timeentry(nextevent, seconds, offset, mode, fun, false));
+	return &gTimeEntries.front();
+}
+
+void *main_timeregister_ms(uint64_t period, void (*fun)(void)) {
+	if (period == 0) {
+		return NULL;
 	}
-	gTimeEntries.push_front(timeentry(nextevent, seconds, offset, mode, fun));
+
+	uint64_t nextevent = usecnow / 1000 + period;
+
+	gTimeEntries.push_front(timeentry(nextevent, period, 0, TIMEMODE_RUN_LATE, fun, true));
 	return &gTimeEntries.front();
 }
 
 void main_timeunregister(void* handler) {
-	TimeEntries::iterator it(gTimeEntries.begin());
-	for (TimeEntries::iterator end(gTimeEntries.end()); it != end; ++it) {
+	for (TimeEntries::iterator it = gTimeEntries.begin(); it != gTimeEntries.end(); ++it) {
 		if (&(*it) == handler) {
 			gTimeEntries.erase(it);
-			break;
+			return;
 		}
 	}
-	massert(it != gTimeEntries.end(), "unregistering unknown handle from time table");
+	mabort("unregistering unknown handle from time table");
 }
 
-int main_timechange(void* x,int mode,uint32_t seconds,uint32_t offset) {
-	timeentry *aux = (timeentry*)x;
-	if (seconds==0 || offset>=seconds) {
+int main_timechange(void* handle, int mode, uint64_t seconds, uint64_t offset) {
+	timeentry *aux = (timeentry*)handle;
+	if (seconds == 0 || offset >= seconds) {
 		return -1;
 	}
-	aux->nextevent = ((now / seconds) * seconds) + offset;
-	while (aux->nextevent<now) {
-		aux->nextevent+=seconds;
-	}
-	aux->seconds = seconds;
+	aux->nextevent = ((now + seconds) / seconds) * seconds + offset;
+	aux->period = seconds;
 	aux->offset = offset;
 	aux->mode = mode;
 	return 0;
 }
 
+int main_timechange_ms(void* handle, uint64_t period) {
+	timeentry *aux = (timeentry*)handle;
+	if (period == 0) {
+		return -1;
+	}
+	aux->nextevent = ((usecnow / 1000 + period) / period) * period;
+	aux->period = period;
+	return 0;
+}
+
 /* internal */
 
 void free_all_registered_entries(void) {
@@ -286,7 +311,6 @@ void free_all_registered_entries(void) {
 	ceentry *ce,*cen;
 	weentry *we,*wen;
 	rlentry *re,*ren;
-	pollentry *pe,*pen;
 	eloopentry *ee,*een;
 
 	for (de = dehead ; de ; de = den) {
@@ -309,10 +333,7 @@ void free_all_registered_entries(void) {
 		free(re);
 	}
 
-	for (pe = pollhead ; pe ; pe = pen) {
-		pen = pe->next;
-		free(pe);
-	}
+	gPollEntries.clear();
 
 	for (ee = eloophead ; ee ; ee = een) {
 		een = ee->next;
@@ -344,10 +365,10 @@ uint8_t main_want_to_terminate() {
 	if (gExitingStatus == ExitingStatus::kRunning) {
 		gExitingStatus = ExitingStatus::kWantExit;
 		syslog(LOG_INFO, "Exiting on internal request.");
-		return STATUS_OK;
+		return LIZARDFS_STATUS_OK;
 	} else {
 		syslog(LOG_ERR, "Unable to exit on internal request.");
-		return ERROR_NOTPOSSIBLE;
+		return LIZARDFS_ERROR_NOTPOSSIBLE;
 	}
 }
 
@@ -358,36 +379,35 @@ void main_want_to_reload() {
 void destruct() {
 	deentry *deit;
 	for (deit = dehead ; deit!=NULL ; deit=deit->next) {
-		deit->fun();
+		try {
+			deit->fun();
+		} catch (Exception& ex) {
+			syslog(LOG_WARNING, "term error: %s", ex.what());
+		}
 	}
 }
 
 void mainloop() {
-	uint32_t prevtime = 0;
+	uint32_t prevtime  = 0;
+	uint64_t prevmtime = 0;
 	struct timeval tv;
-	pollentry *pollit;
 	eloopentry *eloopit;
 	ceentry *ceit;
 	weentry *weit;
 	rlentry *rlit;
-	struct pollfd pdesc[MFSMAXFILES];
-	uint32_t ndesc;
+	std::vector<pollfd> pdesc;
 	int i;
 
 	while (gExitingStatus != ExitingStatus::kDoExit) {
-		ndesc=1;
-		pdesc[0].fd = signalpipe[0];
-		pdesc[0].events = POLLIN;
-		pdesc[0].revents = 0;
-		for (pollit = pollhead ; pollit != NULL ; pollit = pollit->next) {
-			pollit->desc(pdesc,&ndesc);
+		pdesc.clear();
+		pdesc.push_back({signalpipe[0],POLLIN,0});
+		for (auto &pollit: gPollEntries) {
+			pollit.desc(pdesc);
 		}
-		i = poll(pdesc,ndesc, nextPollNonblocking ? 0 : 50);
+		i = poll(pdesc.data(),pdesc.size(), nextPollNonblocking ? 0 : 50);
 		nextPollNonblocking = false;
 		gettimeofday(&tv,NULL);
-		usecnow = tv.tv_sec;
-		usecnow *= 1000000;
-		usecnow += tv.tv_usec;
+		usecnow = tv.tv_sec * uint64_t(1000000) + tv.tv_usec;
 		now = tv.tv_sec;
 		if (i<0) {
 			if (errno==EAGAIN) {
@@ -415,53 +435,78 @@ void mainloop() {
 					}
 				}
 			}
-			for (pollit = pollhead ; pollit != NULL ; pollit = pollit->next) {
-				pollit->serve(pdesc);
+			for (auto &pollit : gPollEntries) {
+				pollit.serve(pdesc);
 			}
 		}
 		for (eloopit = eloophead ; eloopit != NULL ; eloopit = eloopit->next) {
 			eloopit->fun();
 		}
+
+		uint64_t msecnow = usecnow / 1000;
+
+		if (msecnow < prevmtime) {
+			// time went backward - recalculate next event time
+			for (timeentry& timeit : gTimeEntries) {
+				if (!timeit.millisecond_precision) {
+					continue;
+				}
+
+				uint64_t previous_time_to_run = timeit.nextevent - prevmtime;
+				previous_time_to_run = std::min(previous_time_to_run, timeit.period);
+				timeit.nextevent = msecnow + previous_time_to_run;
+			}
+		}
+
 		if (now<prevtime) {
 			// time went backward !!! - recalculate "nextevent" time
 			// adding previous_time_to_run prevents from running next event too soon.
 			for (timeentry& timeit : gTimeEntries) {
-				uint32_t previous_time_to_run = timeit.nextevent - prevtime;
-				if (previous_time_to_run > timeit.seconds) {
-					previous_time_to_run = timeit.seconds;
-				}
-				timeit.nextevent = ((now / timeit.seconds) * timeit.seconds) + timeit.offset;
-				while (timeit.nextevent <= now+previous_time_to_run) {
-					timeit.nextevent += timeit.seconds;
+				if (timeit.millisecond_precision) {
+					continue;
 				}
+
+				uint64_t previous_time_to_run = timeit.nextevent - prevtime;
+				previous_time_to_run = std::min(previous_time_to_run, timeit.period);
+				timeit.nextevent = ((now + previous_time_to_run + timeit.period)
+				                   / timeit.period) * timeit.period + timeit.offset;
 			}
 		} else if (now>prevtime+3600) {
 			// time went forward !!! - just recalculate "nextevent" time
 			for (timeentry& timeit : gTimeEntries) {
-				timeit.nextevent = ((now / timeit.seconds) * timeit.seconds) + timeit.offset;
-				while (now >= timeit.nextevent) {
-					timeit.nextevent += timeit.seconds;
+				if (timeit.millisecond_precision) {
+					timeit.nextevent = msecnow + timeit.period;
+					continue;
 				}
+
+				timeit.nextevent = ((now + timeit.period) / timeit.period)
+				                    * timeit.period + timeit.offset;
 			}
 		}
+
 		for (timeentry& timeit : gTimeEntries) {
+			if (timeit.millisecond_precision) {
+				if (msecnow >= timeit.nextevent) {
+					timeit.nextevent = msecnow + timeit.period;
+					timeit.fun();
+				}
+				continue;
+			}
+
 			if (now >= timeit.nextevent) {
 				if (timeit.mode == TIMEMODE_RUN_LATE) {
-					while (now >= timeit.nextevent) {
-						timeit.nextevent += timeit.seconds;
-					}
 					timeit.fun();
 				} else { /* timeit.mode == TIMEMODE_SKIP_LATE */
 					if (now == timeit.nextevent) {
 						timeit.fun();
 					}
-					while (now >= timeit.nextevent) {
-						timeit.nextevent += timeit.seconds;
-					}
 				}
+				timeit.nextevent += ((now - timeit.nextevent + timeit.period)
+				                    / timeit.period) * timeit.period;
 			}
 		}
-		prevtime = now;
+		prevtime  = now;
+		prevmtime = usecnow / 1000;
 		if (gExitingStatus == ExitingStatus::kRunning && gReloadRequested) {
 			cfg_reload();
 			for (rlit = rlhead ; rlit!=NULL ; rlit=rlit->next) {
@@ -753,6 +798,43 @@ void changeugid(RunMode runmode) {
 	}
 }
 
+bool open_pam_session() {
+#if defined(LIZARDFS_HAVE_PAM)
+	static struct pam_conv conv = {misc_conv, NULL};
+
+	assert(gPAMHandle == NULL);
+
+	int retval;
+
+	retval = pam_start("lizardfs", DEFAULT_USER, &conv, &gPAMHandle);
+	if (retval != PAM_SUCCESS) {
+		lzfs_pretty_errlog(LOG_ERR, "Can't initialize PAM");
+		return false;
+	}
+
+	retval = pam_open_session(gPAMHandle, 0);
+	if (retval != PAM_SUCCESS) {
+		lzfs_pretty_errlog(LOG_ERR, "Can't open PAM session");
+		return false;
+	}
+
+	return true;
+#else
+	return false;
+#endif
+}
+
+void close_pam_session() {
+#if defined(LIZARDFS_HAVE_PAM)
+	if (!gPAMHandle) {
+		return;
+	}
+
+	int retval = pam_close_session(gPAMHandle, 0);
+	pam_end(gPAMHandle, retval);
+#endif
+}
+
 class FileLock {
 public:
 	enum class LockStatus {
@@ -769,18 +851,15 @@ public:
 			  thisProcessCreatedLockFile_(false) {
 		while ((lockstatus_ = wdlock(runmode, timeout)) == LockStatus::kAgain) {
 		}
-		if (lockstatus_ == LockStatus::kFail) {
-			throw FilesystemException("");
-		}
 	}
-	virtual ~FileLock() {
+	~FileLock() {
 		/*
 		 * Order of unlock/file deletion matters
 		 * for RunMode::kRestart mode.
 		 * We need to remove file first and then
 		 * unlock file.
 		 */
-		if (thisProcessCreatedLockFile_) {
+		if (lockstatus_ != LockStatus::kFail && thisProcessCreatedLockFile_) {
 			::unlink(name_.c_str());
 		}
 	}
@@ -795,7 +874,7 @@ public:
 private:
 	LockStatus wdlock(RunMode runmode, uint32_t timeout);
 	pid_t mylock();
-	void createLockFile();
+	bool createLockFile();
 
 	FileDescriptor fd_;
 	std::string name_;
@@ -830,24 +909,32 @@ pid_t FileLock::mylock() {
 	return pid;
 }
 
-void FileLock::createLockFile() {
+bool FileLock::createLockFile() {
 	bool notExisted((::access(name_.c_str(), F_OK) != 0) && (errno == ENOENT));
 	fd_.reset(open(name_.c_str(), O_WRONLY | O_CREAT, 0666));
 	if (!fd_.isOpened()) {
-		throw FilesystemException("can't create lockfile "
-				+ fs::getCurrentWorkingDirectoryNoThrow() + "/" + name_);
+		std::string err = "can't create lockfile " + fs::getCurrentWorkingDirectoryNoThrow()
+		                  + "/" + name();
+		lzfs_pretty_errlog(LOG_ERR, "%s", err.c_str());
+
+		return false;
 	}
 	thisProcessCreatedLockFile_ = notExisted;
+	return true;
 }
 
 FileLock::LockStatus FileLock::wdlock(RunMode runmode, uint32_t timeout) {
 	std::string lockPath = fs::getCurrentWorkingDirectoryNoThrow() + "/" + name_;
-	createLockFile();
+	if(!createLockFile()) {
+		return LockStatus::kFail;
+	}
+
 	pid_t ownerpid(mylock());
 	if (ownerpid<0) {
 		lzfs_pretty_errlog(LOG_ERR, "fcntl error while creating lockfile %s", lockPath.c_str());
 		return LockStatus::kFail;
 	}
+
 	if (ownerpid>0) {
 		if (runmode==RunMode::kIsAlive) {
 			return LockStatus::kAlive;
@@ -1035,6 +1122,15 @@ void makedaemon() {
 	close(STDERR_FILENO);
 	sassert(dup(piped[1])==STDERR_FILENO);
 	close(piped[1]);
+
+	// close all inherited file descriptors
+	int open_max = sysconf(_SC_OPEN_MAX);
+	for (int i = 3; i < open_max; i++) {
+		if (i == signalpipe[0] || i == signalpipe[1]) {
+			continue;
+		}
+		close(i);
+	}
 }
 
 void close_msg_channel() {
@@ -1083,7 +1179,6 @@ void usage(const char *appname) {
 
 int main(int argc,char **argv) {
 	char *wrkdir;
-	char *cfgfile=NULL;
 	char *appname;
 	int ch;
 	int logundefined;
@@ -1091,8 +1186,8 @@ int main(int argc,char **argv) {
 	int32_t nicelevel;
 	uint32_t locktimeout;
 	struct rlimit rls;
-	std::string name_new(ETC_PATH "/mfs/" STR(APPNAME) ".cfg");
-	std::string name_old(ETC_PATH "/" STR(APPNAME) ".cfg");
+	std::string default_cfgfile = ETC_PATH "/" STR(APPNAME) ".cfg";
+	std::string cfgfile = default_cfgfile;
 
 	prepareEnvironment();
 	strerr_init();
@@ -1119,8 +1214,7 @@ int main(int argc,char **argv) {
 				gExtraArguments.emplace_back(optarg);
 				break;
 			case 'c':
-				cfgfile = strdup(optarg);
-				passert(cfgfile);
+				cfgfile = optarg;
 				break;
 			case 'u':
 				logundefined=1;
@@ -1159,20 +1253,6 @@ int main(int argc,char **argv) {
 		return LIZARDFS_EXIT_STATUS_ERROR;
 	}
 
-	if (cfgfile == NULL) { /*try loading default config files*/
-		cfgfile = strdup(name_new.c_str());
-		try {
-			if (!fs::exists(name_new) && fs::exists(name_old)) {
-				free(cfgfile);
-				cfgfile = strdup(name_old.c_str());
-				lzfs_pretty_syslog(LOG_WARNING, "default sysconf path has changed - please move " STR(APPNAME) ".cfg from " ETC_PATH "/ to " ETC_PATH "/mfs/");
-			}
-		} catch (const FilesystemException& ex) {
-			lzfs_pretty_syslog(LOG_WARNING, "can't access config file %s: %s",
-					name_new.c_str(), ex.what());
-		}
-	}
-
 	if (runmode==RunMode::kStart || runmode==RunMode::kRestart) {
 		if (gRunAsDaemon) {
 			makedaemon();
@@ -1181,25 +1261,30 @@ int main(int argc,char **argv) {
 		}
 	}
 
-	if (cfg_load(cfgfile,logundefined)==0) {
+	if (cfg_load(cfgfile.c_str(), logundefined)==0) {
 		lzfs_pretty_syslog(LOG_WARNING, "configuration file %s not found - using defaults; "
 				"please create one to remove this warning "
 				"(you can copy %s.dist to get a base configuration)",
-				cfgfile, name_new.c_str());
+				cfgfile.c_str(), default_cfgfile.c_str());
 	} else if (runmode==RunMode::kStart || runmode==RunMode::kRestart) {
-		lzfs_pretty_syslog(LOG_INFO,"configuration file %s loaded", cfgfile);
+		lzfs_pretty_syslog(LOG_INFO, "configuration file %s loaded", cfgfile.c_str());
 	}
-	free(cfgfile);
 	main_configure_debug_log();
-// const std::string& logappname = set_syslog_ident();
 
 	if (runmode==RunMode::kStart || runmode==RunMode::kRestart) {
-		rls.rlim_cur = MFSMAXFILES;
-		rls.rlim_max = MFSMAXFILES;
-		if (setrlimit(RLIMIT_NOFILE,&rls)<0) {
-			lzfs_pretty_syslog(LOG_WARNING,"can't change open files limit to %u, check restrictions with `ulimit -n'",MFSMAXFILES);
-		} else {
-			lzfs_pretty_syslog(LOG_INFO,"open files limit limit changed to to %u",MFSMAXFILES);
+		if (!open_pam_session()) {
+			rls.rlim_cur = LIZARDFS_MAX_FILES;
+			rls.rlim_max = LIZARDFS_MAX_FILES;
+			if (setrlimit(RLIMIT_NOFILE, &rls) < 0) {
+				lzfs_pretty_syslog(
+				    LOG_WARNING,
+				    "can't change open files limit to %u, check restrictions with `ulimit -n'",
+				    LIZARDFS_MAX_FILES);
+			} else {
+				lzfs_pretty_syslog(
+				    LOG_INFO, "failed to use system open files limit, changed to default value %u",
+				    LIZARDFS_MAX_FILES);
+			}
 		}
 
 		lockmemory = cfg_getnum("LOCK_MEMORY",0);
@@ -1245,16 +1330,9 @@ int main(int argc,char **argv) {
 		return LIZARDFS_EXIT_STATUS_ERROR;
 	}
 
-	std::unique_ptr<FileLock> fl;
-	try {
-		/*
-		 * Only kStart should check for lock file consistency.
-		 */
-		fl.reset(new FileLock(runmode, locktimeout));
-	} catch (const FilesystemException& e) {
-		if (e.what()[0]) {
-			lzfs_pretty_errlog(LOG_ERR, "%s", e.what());
-		}
+	// Only kStart should check for lock file consistency
+	FileLock fl(runmode, locktimeout);
+	if (fl.lockstatus() == FileLock::LockStatus::kFail) {
 		if (gRunAsDaemon) {
 			fputc(0,stderr);
 			close_msg_channel();
@@ -1270,7 +1348,7 @@ int main(int argc,char **argv) {
 		}
 		closelog();
 		if (runmode==RunMode::kIsAlive) {
-			FileLock::LockStatus lockstatus = fl->lockstatus();
+			FileLock::LockStatus lockstatus = fl.lockstatus();
 			sassert((lockstatus == FileLock::LockStatus::kSuccess)
 					|| (lockstatus == FileLock::LockStatus::kAlive));
 			return (lockstatus == FileLock::LockStatus::kAlive
@@ -1313,11 +1391,15 @@ int main(int argc,char **argv) {
 		lzfs_pretty_syslog(LOG_WARNING,"trying to lock memory, but memory lock not supported");
 	}
 #endif
+
 	if (initialize()) {
 		if (getrlimit(RLIMIT_NOFILE,&rls)==0) {
 			lzfs_pretty_syslog(LOG_NOTICE,"open files limit: %lu",(unsigned long)(rls.rlim_cur));
 		}
 		lzfs_pretty_syslog(LOG_NOTICE, STR(APPNAME)" daemon initialized properly");
+#ifdef LIZARDFS_HAVE_SYSTEMD_SD_DAEMON_H
+		sd_notify(0, "READY=1\nSTATUS=" STR(APPNAME) " daemon initialized properly.\n");
+#endif
 		set_syslog_ident();
 		if (gRunAsDaemon) {
 			close_msg_channel();
@@ -1342,5 +1424,6 @@ int main(int argc,char **argv) {
 	cfg_term();
 	strerr_term();
 	closelog();
+	close_pam_session();
 	return ch;
 }
