@@ -1,5 +1,5 @@
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -39,17 +39,18 @@
 #include "common/cwrap.h"
 #include "common/datapack.h"
 #include "common/lizardfs_version.h"
+#include "common/loop_watchdog.h"
 #include "common/main.h"
 #include "common/massert.h"
-#include "common/matoml_communication.h"
 #include "common/metadata.h"
-#include "common/MFSCommunication.h"
-#include "common/mltoma_communication.h"
 #include "common/rotate_files.h"
 #include "common/slogger.h"
 #include "common/sockets.h"
 #include "common/time_utils.h"
 #include "master/changelog.h"
+#include "protocol/matoml.h"
+#include "protocol/MFSCommunication.h"
+#include "protocol/mltoma.h"
 
 #ifndef METALOGGER
 #include "master/filesystem.h"
@@ -324,7 +325,9 @@ namespace {
 }
 
 void masterconn_kill_session(masterconn* eptr) {
-	eptr->mode = KILL;
+	if (eptr->mode != FREE) {
+		eptr->mode = KILL;
+	}
 }
 
 void masterconn_force_metadata_download(masterconn* eptr) {
@@ -426,7 +429,7 @@ void masterconn_metachanges_log(masterconn *eptr,const uint8_t *data,uint32_t le
 
 	if ((lastlogversion > 0) && (version != (lastlogversion + 1))) {
 		syslog(LOG_WARNING, "some changes lost: [%" PRIu64 "-%" PRIu64 "], download metadata again",lastlogversion,version-1);
-		masterconn_handle_changelog_apply_error(eptr, ERROR_METADATAVERSIONMISMATCH);
+		masterconn_handle_changelog_apply_error(eptr, LIZARDFS_ERROR_METADATAVERSIONMISMATCH);
 		return;
 	}
 
@@ -437,7 +440,7 @@ void masterconn_metachanges_log(masterconn *eptr,const uint8_t *data,uint32_t le
 		static char const network[] = "network";
 		uint8_t status;
 		if ((status = restore(network, version, buf.c_str(),
-				RestoreRigor::kDontIgnoreAnyErrors)) != STATUS_OK) {
+				RestoreRigor::kDontIgnoreAnyErrors)) != LIZARDFS_STATUS_OK) {
 			syslog(LOG_WARNING, "malformed changelog sent by the master server, can't apply it. status: %s",
 					mfsstrerr(status));
 			masterconn_handle_changelog_apply_error(eptr, status);
@@ -564,10 +567,10 @@ void masterconn_download_next(masterconn *eptr) {
 						syslog(LOG_WARNING, "can't load downloaded metadata and changelogs: %s",
 								ex.what());
 						uint8_t status = ex.status();
-						if (status == STATUS_OK) {
+						if (status == LIZARDFS_STATUS_OK) {
 							// unknown error - tell the master to apply changelogs and hope that
 							// all will be good
-							status = ERROR_CHANGELOGINCONSISTENT;
+							status = LIZARDFS_ERROR_CHANGELOGINCONSISTENT;
 						}
 						masterconn_handle_changelog_apply_error(eptr, status);
 					}
@@ -600,6 +603,10 @@ void masterconn_download_start(masterconn *eptr,const uint8_t *data,uint32_t len
 		syslog(LOG_NOTICE,"download start error");
 		return;
 	}
+#ifndef METALOGGER
+	// We are a shadow master and we are going to do some changes in the data dir right now
+	fs_erase_message_from_lockfile();
+#endif
 	eptr->filesize = get64bit(&data);
 	eptr->dloffset = 0;
 	eptr->downloadretrycnt = 0;
@@ -703,9 +710,9 @@ void masterconn_changelog_apply_error(masterconn *eptr, const uint8_t *data, uin
 	uint8_t status;
 	matoml::changelogApplyError::deserialize(data, length, status);
 	DEBUG_LOG("master.matoml_changelog_apply_error") << "status: " << int(status);
-	if (status == STATUS_OK) {
+	if (status == LIZARDFS_STATUS_OK) {
 		masterconn_force_metadata_download(eptr);
-	} else if (status == ERROR_DELAYED) {
+	} else if (status == LIZARDFS_ERROR_DELAYED) {
 		eptr->state = MasterConnectionState::kLimbo;
 		syslog(LOG_NOTICE, "Master temporarily refused to produce a new metadata image");
 	} else {
@@ -889,10 +896,13 @@ void masterconn_connecttest(masterconn *eptr) {
 }
 
 void masterconn_read(masterconn *eptr) {
+	SignalLoopWatchdog watchdog;
 	int32_t i;
 	uint32_t type,size;
 	const uint8_t *ptr;
-	for (;;) {
+
+	watchdog.start();
+	while (eptr->mode != KILL) {
 		i=read(eptr->sock,eptr->inputpacket.startptr,eptr->inputpacket.bytesleft);
 		if (i==0) {
 			syslog(LOG_NOTICE,"connection was reset by Master");
@@ -951,16 +961,18 @@ void masterconn_read(masterconn *eptr) {
 			eptr->inputpacket.packet=NULL;
 		}
 
-		if (eptr->mode==KILL) {
-			// masterconn_gotpacket killed us
-			return;
+		if (watchdog.expired()) {
+			break;
 		}
 	}
 }
 
 void masterconn_write(masterconn *eptr) {
+	SignalLoopWatchdog watchdog;
 	packetstruct *pack;
 	int32_t i;
+
+	watchdog.start();
 	for (;;) {
 		pack = eptr->outputhead;
 		if (pack==NULL) {
@@ -986,15 +998,27 @@ void masterconn_write(masterconn *eptr) {
 			eptr->outputtail = &(eptr->outputhead);
 		}
 		free(pack);
+
+		if (watchdog.expired()) {
+			break;
+		}
 	}
 }
 
+void masterconn_wantexit(void) {
+	if (masterconnsingleton) {
+		masterconn_kill_session(masterconnsingleton);
+	}
+}
 
-void masterconn_desc(struct pollfd *pdesc,uint32_t *ndesc) {
+int masterconn_canexit(void) {
+	return !masterconnsingleton || masterconnsingleton->mode == FREE;
+}
+
+void masterconn_desc(std::vector<pollfd> &pdesc) {
 	if (!masterconnsingleton) {
 		return;
 	}
-	uint32_t pos = *ndesc;
 	masterconn *eptr = masterconnsingleton;
 
 	eptr->pdescpos = -1;
@@ -1002,25 +1026,20 @@ void masterconn_desc(struct pollfd *pdesc,uint32_t *ndesc) {
 		return;
 	}
 	if (eptr->mode==HEADER || eptr->mode==DATA) {
-		pdesc[pos].fd = eptr->sock;
-		pdesc[pos].events = POLLIN;
-		eptr->pdescpos = pos;
-		pos++;
+		pdesc.push_back({eptr->sock,POLLIN,0});
+		eptr->pdescpos = pdesc.size() - 1;
 	}
 	if (((eptr->mode==HEADER || eptr->mode==DATA) && eptr->outputhead!=NULL) || eptr->mode==CONNECTING) {
 		if (eptr->pdescpos>=0) {
 			pdesc[eptr->pdescpos].events |= POLLOUT;
 		} else {
-			pdesc[pos].fd = eptr->sock;
-			pdesc[pos].events = POLLOUT;
-			eptr->pdescpos = pos;
-			pos++;
+			pdesc.push_back({eptr->sock,POLLOUT,0});
+			eptr->pdescpos = pdesc.size() - 1;
 		}
 	}
-	*ndesc = pos;
 }
 
-void masterconn_serve(struct pollfd *pdesc) {
+void masterconn_serve(const std::vector<pollfd> &pdesc) {
 	if (!masterconnsingleton) {
 		return;
 	}
@@ -1060,8 +1079,10 @@ void masterconn_serve(struct pollfd *pdesc) {
 	if (eptr->mode == KILL) {
 		masterconn_beforeclose(eptr);
 		tcpclose(eptr->sock);
+		eptr->sock = -1;
 		if (eptr->inputpacket.packet) {
 			free(eptr->inputpacket.packet);
+			eptr->inputpacket.packet = NULL;
 		}
 		pptr = eptr->outputhead;
 		while (pptr) {
@@ -1072,6 +1093,7 @@ void masterconn_serve(struct pollfd *pdesc) {
 			pptr = pptr->next;
 			free(paptr);
 		}
+		eptr->outputhead = NULL;
 		eptr->mode = FREE;
 		eptr->version = 0;
 	}
@@ -1197,6 +1219,7 @@ int masterconn_init(void) {
 	eptr->pdescpos = -1;
 	eptr->metafd = -1;
 	eptr->version = 0;
+	eptr->sock  = -1;
 	eptr->state = MasterConnectionState::kNone;
 #ifdef METALOGGER
 	changelogsMigrateFrom_1_6_29("changelog_ml");
@@ -1212,6 +1235,8 @@ int masterconn_init(void) {
 	main_destructregister(masterconn_term);
 	main_pollregister(masterconn_desc,masterconn_serve);
 	main_reloadregister(masterconn_reload);
+	main_wantexitregister(masterconn_wantexit);
+	main_canexitregister(masterconn_canexit);
 #ifndef METALOGGER
 	metadataserver::registerFunctionCalledOnPromotion(masterconn_become_master);
 #endif
