@@ -1,6 +1,6 @@
 
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -38,16 +38,17 @@
 #include "common/crc.h"
 #include "common/datapack.h"
 #include "common/lizardfs_version.h"
+#include "common/loop_watchdog.h"
 #include "common/main.h"
 #include "common/massert.h"
-#include "common/matoml_communication.h"
 #include "common/metadata.h"
-#include "common/mltoma_communication.h"
-#include "common/MFSCommunication.h"
 #include "common/slogger.h"
 #include "common/sockets.h"
 #include "master/filesystem.h"
 #include "master/personality.h"
+#include "protocol/matoml.h"
+#include "protocol/MFSCommunication.h"
+#include "protocol/mltoma.h"
 
 #define MaxPacketSize 1500000
 #define OLD_CHANGES_BLOCK_SIZE 5000
@@ -426,7 +427,7 @@ void matomlserv_register_shadow(matomlserventry *eptr, const uint8_t *data, uint
 		syslog(LOG_NOTICE,
 				"MLTOMA_REGISTER_SHADOW - rejected old client (v%s) from %s",
 				lizardfsVersionToString(eptr->version).c_str(), eptr->servstrip);
-		matomlserv_createpacket(eptr, matoml::registerShadow::build(uint8_t(ERROR_REGISTER)));
+		matomlserv_createpacket(eptr, matoml::registerShadow::build(uint8_t(LIZARDFS_ERROR_REGISTER)));
 		return;
 	}
 
@@ -542,12 +543,7 @@ void matomlserv_download_data(matomlserventry *eptr,const uint8_t *data,uint32_t
 	ptr = matomlserv_createpacket(eptr,MATOML_DOWNLOAD_DATA,16+leng);
 	put64bit(&ptr,offset);
 	put32bit(&ptr,leng);
-#ifdef LIZARDFS_HAVE_PREAD
 	ret = pread(eptr->metafd,ptr+4,leng,offset);
-#else /* LIZARDFS_HAVE_PWRITE */
-	lseek(eptr->metafd,offset,SEEK_SET);
-	ret = read(eptr->metafd,ptr+4,leng);
-#endif /* LIZARDFS_HAVE_PWRITE */
 	if (ret!=(ssize_t)leng) {
 		lzfs_silent_errlog(LOG_NOTICE,"error reading metafile");
 		eptr->mode=KILL;
@@ -587,7 +583,7 @@ void matomlserv_changelog_apply_error(matomlserventry *eptr, const uint8_t *data
 				mfsstrerr(recvStatus));
 		gShadowQueue.addRequest(eptr);
 		fs_storeall(MetadataDumper::kBackgroundDump);
-		if (recvStatus == ERROR_BADMETADATACHECKSUM) {
+		if (recvStatus == LIZARDFS_ERROR_BADMETADATACHECKSUM) {
 			fs_start_checksum_recalculation();
 		}
 	} else {
@@ -597,7 +593,7 @@ void matomlserv_changelog_apply_error(matomlserventry *eptr, const uint8_t *data
 				"refusing to store metadata because only %" PRIi32 " seconds elapsed since the "
 				"previous request and METADATA_SAVE_REQUEST_MIN_PERIOD=%" PRIu32,
 				mfsstrerr(recvStatus), secondsSinceLastRequest, gMinMetadataSaveRequestPeriod_s);
-		matomlserv_createpacket(eptr, matoml::changelogApplyError::build(ERROR_DELAYED));
+		matomlserv_createpacket(eptr, matoml::changelogApplyError::build(LIZARDFS_ERROR_DELAYED));
 	}
 }
 
@@ -720,10 +716,13 @@ void matomlserv_term(void) {
 }
 
 void matomlserv_read(matomlserventry *eptr) {
+	SignalLoopWatchdog watchdog;
 	int32_t i;
 	uint32_t type,size;
 	const uint8_t *ptr;
-	for (;;) {
+
+	watchdog.start();
+	while (eptr->mode != KILL) {
 		i=read(eptr->sock,eptr->inputpacket.startptr,eptr->inputpacket.bytesleft);
 		if (i==0) {
 			syslog(LOG_NOTICE,"connection with ML(%s) has been closed by peer",eptr->servstrip);
@@ -779,13 +778,21 @@ void matomlserv_read(matomlserventry *eptr) {
 				free(eptr->inputpacket.packet);
 			}
 			eptr->inputpacket.packet=NULL;
+			break;
+		}
+
+		if (watchdog.expired()) {
+			break;
 		}
 	}
 }
 
 void matomlserv_write(matomlserventry *eptr) {
+	SignalLoopWatchdog watchdog;
 	packetstruct *pack;
 	int32_t i;
+
+	watchdog.start();
 	for (;;) {
 		pack = eptr->outputhead;
 		if (pack==NULL) {
@@ -810,33 +817,31 @@ void matomlserv_write(matomlserventry *eptr) {
 			eptr->outputtail = &(eptr->outputhead);
 		}
 		free(pack);
+
+		if (watchdog.expired()) {
+			break;
+		}
 	}
 }
 
-void matomlserv_desc(struct pollfd *pdesc,uint32_t *ndesc) {
-	uint32_t pos = *ndesc;
+void matomlserv_desc(std::vector<pollfd> &pdesc) {
 	matomlserventry *eptr;
 	if (!gExiting) {
-		pdesc[pos].fd = lsock;
-		pdesc[pos].events = POLLIN;
-		lsockpdescpos = pos;
-		pos++;
+		pdesc.push_back({lsock,POLLIN,0});
+		lsockpdescpos = pdesc.size() - 1;
 	} else {
 		lsockpdescpos = -1;
 	}
 	for (eptr=matomlservhead ; eptr ; eptr=eptr->next) {
-		pdesc[pos].fd = eptr->sock;
-		pdesc[pos].events = POLLIN;
-		eptr->pdescpos = pos;
+		pdesc.push_back({eptr->sock,POLLIN,0});
+		eptr->pdescpos = pdesc.size() - 1;
 		if (eptr->outputhead!=NULL) {
-			pdesc[pos].events |= POLLOUT;
+			pdesc.back().events |= POLLOUT;
 		}
-		pos++;
 	}
-	*ndesc = pos;
 }
 
-void matomlserv_serve(struct pollfd *pdesc) {
+void matomlserv_serve(const std::vector<pollfd> &pdesc) {
 	uint32_t now=main_time();
 	matomlserventry *eptr,**kptr;
 	packetstruct *pptr,*paptr;
