@@ -1,5 +1,5 @@
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS.
 
@@ -24,9 +24,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common/MFSCommunication.h"
+#include "protocol/MFSCommunication.h"
 #include "common/slogger.h"
 #include "master/filesystem.h"
+#include "master/filesystem_snapshot.h"
+#include "master/filesystem_operations.h"
 
 #ifndef METARESTORE
 # include "common/debug_log.h"
@@ -189,8 +191,8 @@
 	} \
 }
 
-#define GETU32(data,clptr) do { char* end = NULL; (data)=strtoul(clptr,&end,10); clptr = end; } while (false)
-#define GETU64(data,clptr) do { char* end = NULL; (data)=strtoull(clptr,&end,10); clptr = end; } while (false)
+#define GETU32(data,clptr) do { char* end_ = NULL; (data)=strtoul(clptr,&end_,10); clptr = end_; } while (0)
+#define GETU64(data,clptr) do { char* end_ = NULL; (data)=strtoull(clptr,&end_,10); clptr = end_; } while (0)
 
 int do_access(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
 	uint32_t inode;
@@ -270,7 +272,7 @@ int do_create(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
 	EAT(ptr,filename,lv,')');
 	EAT(ptr,filename,lv,':');
 	GETU32(inode,ptr);
-	return fs_apply_create(ts,parent,strlen((char*)name),name,type,mode,uid,gid,rdev,inode);
+	return fs_apply_create(ts, parent, HString((const char*)name), type, mode, uid, gid, rdev, inode);
 }
 
 int do_session(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
@@ -283,7 +285,7 @@ int do_session(const char *filename, uint64_t lv, uint32_t ts, const char *ptr)
 	return fs_apply_session(cuid);
 }
 
-int do_emptytrash(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+int do_emptytrash_deprecated(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
 	uint32_t reservedinodes,freeinodes;
 	EAT(ptr,filename,lv,'(');
 	EAT(ptr,filename,lv,')');
@@ -291,16 +293,16 @@ int do_emptytrash(const char *filename, uint64_t lv, uint32_t ts, const char *pt
 	GETU32(freeinodes,ptr);
 	EAT(ptr,filename,lv,',');
 	GETU32(reservedinodes,ptr);
-	return fs_apply_emptytrash(ts,freeinodes,reservedinodes);
+	return fs_apply_emptytrash_deprecated(ts,freeinodes,reservedinodes);
 }
 
-int do_emptyreserved(const char *filename, uint64_t lv, uint32_t ts, const char* ptr) {
+int do_emptyreserved_deprecated(const char *filename, uint64_t lv, uint32_t ts, const char* ptr) {
 	uint32_t freeinodes;
 	EAT(ptr,filename,lv,'(');
 	EAT(ptr,filename,lv,')');
 	EAT(ptr,filename,lv,':');
 	GETU32(freeinodes,ptr);
-	return fs_apply_emptyreserved(ts,freeinodes);
+	return fs_apply_emptyreserved_deprecated(ts,freeinodes);
 }
 
 int do_freeinodes(const char *filename, uint64_t lv, uint32_t ts, const char* ptr) {
@@ -331,7 +333,7 @@ int do_link(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
 	EAT(ptr,filename,lv,',');
 	GETNAME(name,ptr,filename,lv,')');
 	EAT(ptr,filename,lv,')');
-	return fs_link(FsContext::getForRestore(ts), inode, parent, strlen((char*)name), name,
+	return fs_link(FsContext::getForRestore(ts), inode, parent, HString((const char*)name),
 			nullptr, nullptr);
 }
 
@@ -361,11 +363,107 @@ int do_move(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
 	EAT(ptr,filename,lv,':');
 	GETU32(inode,ptr);
 	return fs_rename(FsContext::getForRestore(ts),
-			parent_src, strlen((char*)name_src), name_src,
-			parent_dst, strlen((char*)name_dst), name_dst,
+			parent_src, HString((const char*)name_src),
+			parent_dst, HString((const char*)name_dst),
 			&inode, nullptr);
 }
 
+int do_lock_op(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+	uint32_t lock_type, inode, sessionid;
+	uint64_t start, end;
+	uint64_t owner;
+	uint32_t op;
+	const bool nonblocking = false;
+	std::vector<FileLocks::Owner> dummy_applied;
+	EAT(ptr, filename, lv, '(');
+	GETU32(lock_type, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(inode, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU64(start, ptr);
+	EAT(ptr ,filename, lv, ',');
+	GETU64(end, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU64(owner, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(sessionid, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(op, ptr);
+	EAT(ptr,filename,lv,')');
+
+	int status = LIZARDFS_STATUS_OK;
+
+	switch (static_cast<lzfs_locks::Type>(lock_type)) {
+	case lzfs_locks::Type::kFlock:
+		status = fs_flock_op(FsContext::getForRestore(ts), inode, owner, sessionid, 0, 0,
+				op, nonblocking, dummy_applied);
+		break;
+	case lzfs_locks::Type::kPosix:
+		status = fs_posixlock_op(FsContext::getForRestore(ts), inode, start, end, owner, sessionid, 0, 0,
+				op, nonblocking, dummy_applied);
+		break;
+	default:
+		lzfs_pretty_syslog(LOG_ERR, "Invalid lock type passed to restore: %u", lock_type);
+		return LIZARDFS_ERROR_EINVAL;
+	}
+
+	if (status==LIZARDFS_ERROR_WAITING) {
+		return LIZARDFS_STATUS_OK;
+	}
+	return status;
+}
+
+int do_remove_pending_op(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+	uint32_t lock_type;
+	uint64_t ownerid;
+	uint32_t sessionid;
+	uint32_t inode;
+	uint64_t reqid;
+
+	EAT(ptr, filename, lv, '(');
+	GETU32(lock_type, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU64(ownerid, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(sessionid, ptr);
+	EAT(ptr ,filename, lv, ',');
+	GETU32(inode, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU64(reqid, ptr);
+	EAT(ptr,filename,lv,')');
+
+	return fs_locks_remove_pending(FsContext::getForRestore(ts), lock_type, ownerid, sessionid,
+		inode, reqid);
+}
+
+int do_lock_clear_session(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+	uint32_t lock_type, inode, sessionid;
+	std::vector<FileLocks::Owner> applied;
+
+	EAT(ptr, filename, lv, '(');
+	GETU32(lock_type, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(inode, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(sessionid, ptr);
+	EAT(ptr, filename, lv, ')');
+
+	return fs_locks_clear_session(FsContext::getForRestore(ts), lock_type, inode, sessionid, applied);
+}
+
+int do_lock_unlock_inode(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+	uint32_t lock_type, inode;
+	std::vector<FileLocks::Owner> applied;
+
+	EAT(ptr, filename, lv, '(');
+	GETU32(lock_type, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(inode, ptr);
+	EAT(ptr, filename, lv, ')');
+
+	return fs_locks_unlock_inode(FsContext::getForRestore(ts), lock_type, inode, applied);
+}
+
 int do_purge(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
 	uint32_t inode;
 	EAT(ptr,filename,lv,'(');
@@ -418,26 +516,36 @@ int do_seteattr(const char* filename, uint64_t lv, uint32_t ts, const char* ptr)
 	return fs_seteattr(FsContext::getForRestoreWithUidGid(ts, uid, 0), inode, eattr, smode, &ci, &nci, &npi);
 }
 
-int do_setgoal(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
-	uint32_t inode,uid,ci,nci,npi;
-	uint8_t goal,smode;
-	EAT(ptr,filename,lv,'(');
-	GETU32(inode,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(uid,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(goal,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(smode,ptr);
-	EAT(ptr,filename,lv,')');
-	EAT(ptr,filename,lv,':');
-	GETU32(ci,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(nci,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(npi,ptr);
-	return fs_setgoal(FsContext::getForRestoreWithUidGid(ts, uid, 0),
-			inode, goal, smode, &ci, &nci, &npi);
+int do_setgoal(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+	uint32_t inode, uid, ci, nci, npi;
+	uint8_t goal, smode;
+	EAT(ptr, filename, lv, '(');
+	GETU32(inode, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(uid, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(goal, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(smode, ptr);
+	EAT(ptr, filename, lv, ')');
+	if (*(ptr) == ':') {
+		EAT(ptr, filename, lv, ':');
+		GETU32(ci, ptr);
+		if (*(ptr) == ',') {
+			EAT(ptr, filename, lv, ',');
+			GETU32(nci, ptr);
+			EAT(ptr, filename, lv, ',');
+			GETU32(npi, ptr);
+			return fs_deprecated_setgoal(FsContext::getForRestoreWithUidGid(ts, uid, 0),
+			                             inode, goal, smode, &ci, &nci, &npi);
+		} else {
+			return fs_apply_setgoal(FsContext::getForRestoreWithUidGid(ts, uid, 0),
+			                        inode, goal, smode, ci);
+		}
+	} else {
+		return fs_apply_setgoal(FsContext::getForRestoreWithUidGid(ts, uid, 0), inode, goal,
+		                        smode, SetGoalTask::kChanged);
+	}
 }
 
 int do_setpath(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
@@ -449,29 +557,41 @@ int do_setpath(const char* filename, uint64_t lv, uint32_t ts, const char* ptr)
 	EAT(ptr,filename,lv,',');
 	GETPATH(path,pathsize,ptr,filename,lv,')');
 	EAT(ptr,filename,lv,')');
-	return fs_settrashpath(FsContext::getForRestore(ts), inode, strlen((const char*)path), path);
+	return fs_settrashpath(FsContext::getForRestore(ts), inode, std::string((const char*)path));
 }
 
-int do_settrashtime(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
-	uint32_t inode,uid,ci,nci,npi;
+int do_settrashtime(const char *filename, uint64_t lv, uint32_t ts, const char *ptr) {
+	uint32_t inode, uid, ci, nci, npi;
 	uint32_t trashtime;
 	uint8_t smode;
-	EAT(ptr,filename,lv,'(');
-	GETU32(inode,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(uid,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(trashtime,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(smode,ptr);
-	EAT(ptr,filename,lv,')');
-	EAT(ptr,filename,lv,':');
-	GETU32(ci,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(nci,ptr);
-	EAT(ptr,filename,lv,',');
-	GETU32(npi,ptr);
-	return fs_settrashtime(FsContext::getForRestoreWithUidGid(ts, uid, 0), inode, trashtime, smode, &ci, &nci, &npi);
+	EAT(ptr, filename, lv, '(');
+	GETU32(inode, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(uid, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(trashtime, ptr);
+	EAT(ptr, filename, lv, ',');
+	GETU32(smode, ptr);
+	EAT(ptr, filename, lv, ')');
+	if ((*ptr) == ':') {
+		EAT(ptr, filename, lv, ':');
+		GETU32(ci, ptr);
+		if ((*ptr) == ',') {
+			EAT(ptr, filename, lv, ',');
+			GETU32(nci, ptr);
+			EAT(ptr, filename, lv, ',');
+			GETU32(npi, ptr);
+			return fs_deprecated_settrashtime(
+			        FsContext::getForRestoreWithUidGid(ts, uid, 0), inode, trashtime,
+			        smode, &ci, &nci, &npi);
+		} else {
+			return fs_apply_settrashtime(FsContext::getForRestoreWithUidGid(ts, uid, 0),
+			                             inode, trashtime, smode, ci);
+		}
+	} else {
+		return fs_apply_settrashtime(FsContext::getForRestoreWithUidGid(ts, uid, 0), inode,
+		                             trashtime, smode, SetTrashtimeTask::kChanged);
+	}
 }
 
 int do_setxattr(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
@@ -561,8 +681,27 @@ int do_snapshot(const char* filename, uint64_t lv, uint32_t ts, const char* ptr)
 	EAT(ptr,filename,lv,',');
 	GETU32(canoverwrite,ptr);
 	EAT(ptr,filename,lv,')');
-	return fs_snapshot(FsContext::getForRestore(ts),
-			inode, parent, strlen((char*)name), name, canoverwrite);
+
+	return fs_deprecated_snapshot(FsContext::getForRestore(ts),
+			inode, parent, HString((const char*)name), canoverwrite);
+}
+
+int do_clone_node(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
+	uint32_t src_inode, dst_parent, dst_inode, can_overwrite;
+	uint8_t name[256];
+	EAT(ptr,filename,lv,'(');
+	GETU32(src_inode,ptr);
+	EAT(ptr,filename,lv,',');
+	GETU32(dst_parent,ptr);
+	EAT(ptr,filename,lv,',');
+	GETU32(dst_inode,ptr);
+	EAT(ptr,filename,lv,',');
+	GETNAME(name,ptr,filename,lv,',');
+	EAT(ptr,filename,lv,',');
+	GETU32(can_overwrite,ptr);
+	EAT(ptr,filename,lv,')');
+	return fs_clone_node(FsContext::getForRestore(ts), src_inode, dst_parent, dst_inode,
+				HString((const char*)name), can_overwrite);
 }
 
 int do_symlink(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
@@ -584,7 +723,7 @@ int do_symlink(const char* filename, uint64_t lv, uint32_t ts, const char* ptr)
 	EAT(ptr,filename,lv,':');
 	GETU32(inode,ptr);
 	return fs_symlink(FsContext::getForRestoreWithUidGid(ts, uid, gid),
-			parent, strlen((char*)name), name, strlen((char*)path), path, &inode, nullptr);
+			parent, HString((char*)name), std::string((char*)path), &inode, nullptr);
 }
 
 int do_undel(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
@@ -605,7 +744,7 @@ int do_unlink(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
 	EAT(ptr,filename,lv,')');
 	EAT(ptr,filename,lv,':');
 	GETU32(inode,ptr);
-	return fs_apply_unlink(ts,parent,strlen((char*)name),name,inode);
+	return fs_apply_unlink(ts, parent, HString((char*)name), inode);
 }
 
 int do_unlock(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
@@ -627,21 +766,28 @@ int do_nextchunkid(const char* filename, uint64_t lv, uint32_t ts, const char* p
 
 
 int do_trunc(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
-	uint32_t inode,indx;
+	uint32_t inode,indx,lockid;
 	uint64_t chunkid;
 	EAT(ptr,filename,lv,'(');
 	GETU32(inode,ptr);
 	EAT(ptr,filename,lv,',');
 	GETU32(indx,ptr);
+	if (*ptr==',') {
+		EAT(ptr,filename,lv,',');
+		GETU32(lockid,ptr);
+	} else {
+		lockid = 0;
+	}
 	EAT(ptr,filename,lv,')');
 	EAT(ptr,filename,lv,':');
 	GETU64(chunkid,ptr);
-	return fs_apply_trunc(ts,inode,indx,chunkid);
+	return fs_apply_trunc(ts,inode,indx,chunkid,lockid);
 }
 
 int do_write(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
 	uint32_t inode,indx;
 	uint64_t chunkid;
+	uint32_t lockid;
 	uint8_t opflag;
 	EAT(ptr,filename,lv,'(');
 	GETU32(inode,ptr);
@@ -653,19 +799,24 @@ int do_write(const char* filename, uint64_t lv, uint32_t ts, const char* ptr) {
 	} else {
 		opflag=1;
 	}
+	if (*ptr==',') {
+		EAT(ptr,filename,lv,',');
+		GETU32(lockid,ptr);
+	} else {
+		lockid=1;
+	}
 	EAT(ptr,filename,lv,')');
 	EAT(ptr,filename,lv,':');
 	GETU64(chunkid,ptr);
-	return fs_writechunk(FsContext::getForRestore(ts), inode, indx, &chunkid, &opflag, nullptr);
+	return fs_writechunk(FsContext::getForRestore(ts), inode, indx, false, &lockid, &chunkid, &opflag, nullptr);
 }
 
-
 int restore_line(const char* filename, uint64_t lv, const char* line) {
 	uint32_t ts;
 	int status;
-	const char* errormsgs[] = {ERROR_STRINGS};
+	const char* errormsgs[] = {LIZARDFS_ERROR_STRINGS};
 
-	status = ERROR_MAX;
+	status = LIZARDFS_ERROR_MAX;
 	const char* ptr = line;
 
 	EAT(ptr,filename,lv,':');
@@ -689,10 +840,14 @@ int restore_line(const char* filename, uint64_t lv, const char* line) {
 		case 'C':
 			if (strncmp(ptr,"CHECKSUM",8)==0) {
 				status = do_checksum(filename,lv,ts,ptr+8);
+			} else if (strncmp(ptr,"CLONE",5)==0) {
+				status = do_clone_node(filename,lv,ts,ptr+5);
 			} else if (strncmp(ptr,"CREATE",6)==0) {
 				status = do_create(filename,lv,ts,ptr+6);
 			} else if (strncmp(ptr,"CUSTOMER",8)==0) {      // deprecated
 				status = do_session(filename,lv,ts,ptr+8);
+			} else if (strncmp(ptr,"CLRLCK",6)==0) {
+				status = do_lock_clear_session(filename,lv,ts,ptr+6);
 			}
 			break;
 		case 'D':
@@ -702,13 +857,17 @@ int restore_line(const char* filename, uint64_t lv, const char* line) {
 			break;
 		case 'E':
 			if (strncmp(ptr,"EMPTYTRASH",10)==0) {
-				status = do_emptytrash(filename,lv,ts,ptr+10);
+				status = do_emptytrash_deprecated(filename,lv,ts,ptr+10);
 			} else if (strncmp(ptr,"EMPTYRESERVED",13)==0) {
-				status = do_emptyreserved(filename,lv,ts,ptr+13);
+				status = do_emptyreserved_deprecated(filename,lv,ts,ptr+13);
 			}
 			break;
 		case 'F':
-			if (strncmp(ptr,"FREEINODES",10)==0) {
+			if (strncmp(ptr,"FLCKINODE",9)==0) {
+				status = do_lock_unlock_inode(filename,lv,ts,ptr+9);
+			} else if (strncmp(ptr, "FLCK", 4) == 0) {
+				status = do_lock_op(filename,lv,ts,ptr+4);
+			} else if (strncmp(ptr, "FREEINODES", 10) == 0) {
 				status = do_freeinodes(filename,lv,ts,ptr+10);
 			}
 			break;
@@ -744,6 +903,8 @@ int restore_line(const char* filename, uint64_t lv, const char* line) {
 				status = do_release(filename,lv,ts,ptr+7);
 			} else if (strncmp(ptr,"REPAIR",6)==0) {
 				status = do_repair(filename,lv,ts,ptr+6);
+			} else if (strncmp(ptr,"RMPLOCK",7)==0) {
+				status = do_remove_pending_op(filename,lv,ts,ptr+7);
 			}
 			break;
 		case 'S':
@@ -763,7 +924,7 @@ int restore_line(const char* filename, uint64_t lv, const char* line) {
 				status = do_settrashtime(filename,lv,ts,ptr+12);
 			} else if (strncmp(ptr,"SETXATTR",8)==0) {
 				status = do_setxattr(filename,lv,ts,ptr+8);
-			} else if (strncmp(ptr,"SNAPSHOT",8)==0) {
+			} else if (strncmp(ptr,"SNAPSHOT",8)==0) {    // deprecated
 				status = do_snapshot(filename,lv,ts,ptr+8);
 			} else if (strncmp(ptr,"SYMLINK",7)==0) {
 				status = do_symlink(filename,lv,ts,ptr+7);
@@ -792,13 +953,13 @@ int restore_line(const char* filename, uint64_t lv, const char* line) {
 			break;
 	}
 
-	if (status == ERROR_MAX) {
+	if (status == LIZARDFS_ERROR_MAX) {
 #ifndef METARESTORE
 		DEBUG_LOG("master.mismatch")
 				<< "File " << filename << ", " << lv << line << " -- unknown entry ";
 #endif
 		lzfs_pretty_syslog(LOG_ERR, "%s:%" PRIu64 ": unknown entry '%s'",filename,lv,ptr);
-	} else if (status != STATUS_OK) {
+	} else if (status != LIZARDFS_STATUS_OK) {
 #ifndef METARESTORE
 		DEBUG_LOG("master.mismatch")
 				<< "File " << filename << ", " << lv << line << " -- " << mfsstrerr(status);
@@ -848,7 +1009,7 @@ uint8_t restore(const char* filename, uint64_t newLogVersion, const char *ptr, R
 				"merge error - possibly corrupted input file - ignore entry"
 				" (filename: %s, versions: %" PRIu64 ", %" PRIu64 ")",
 				filename, newLogVersion, currentFsVersion);
-		return STATUS_OK;
+		return LIZARDFS_STATUS_OK;
 	} else if (newLogVersion >= nextFsVersion) {
 		if (newLogVersion == currentFsVersion) {
 			if (verbosity > 1) {
@@ -858,16 +1019,16 @@ uint8_t restore(const char* filename, uint64_t newLogVersion, const char *ptr, R
 		} else if (newLogVersion > currentFsVersion + 1) {
 			lzfs_pretty_syslog(LOG_ERR, "hole in change files (entries from %s:%" PRIu64 " to %s:%" PRIu64
 					" are missing) - add more files", lastfn, currentFsVersion + 1, filename, newLogVersion - 1);
-			return ERROR_CHANGELOGINCONSISTENT;
+			return LIZARDFS_ERROR_CHANGELOGINCONSISTENT;
 		} else {
 			if (verbosity > 0) {
 				lzfs_pretty_syslog(LOG_NOTICE, "%s: change %s", filename, ptr);
 			}
 			int status = restore_line(filename,newLogVersion,ptr);
 			if (status<0) { // parse error - stop processing if requested
-				return (rigor == RestoreRigor::kIgnoreParseErrors ? 0 : ERROR_PARSE);
+				return (rigor == RestoreRigor::kIgnoreParseErrors ? 0 : LIZARDFS_ERROR_PARSE);
 			}
-			if (status != STATUS_OK) { // other errors - stop processing data
+			if (status != LIZARDFS_STATUS_OK) { // other errors - stop processing data
 				return status;
 			}
 			nextFsVersion = fs_getversion();
@@ -876,13 +1037,13 @@ uint8_t restore(const char* filename, uint64_t newLogVersion, const char *ptr, R
 				 * restore_line() should bump nextFsVersion by exactly 1, but it didn't.
 				 */
 				lzfs_pretty_syslog(LOG_ERR, "%s:%" PRIu64 ":%" PRIu64 " version mismatch", filename, newLogVersion, nextFsVersion);
-				return ERROR_METADATAVERSIONMISMATCH;
+				return LIZARDFS_ERROR_METADATAVERSIONMISMATCH;
 			}
 		}
 	}
 	currentFsVersion = newLogVersion;
 	lastfn = filename;
-	return STATUS_OK;
+	return LIZARDFS_STATUS_OK;
 }
 
 void restore_setverblevel(uint8_t _vlevel) {
