@@ -1,5 +1,5 @@
 /*
-   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013 Skytechnology sp. z o.o..
+   Copyright 2005-2010 Jakub Kruszona-Zawadzki, Gemius SA, 2013-2014 EditShare, 2013-2015 Skytechnology sp. z o.o..
 
    This file was part of MooseFS and is part of LizardFS
 
@@ -25,12 +25,15 @@
 #include <vector>
 
 #include "common/massert.h"
-#include "common/MFSCommunication.h"
+#include "common/special_inode_defs.h"
+#include "mount/fuse/lock_conversion.h"
 #include "mount/lizard_client.h"
 #include "mount/lizard_client_context.h"
+#include "mount/thread_safe_map.h"
+#include "protocol/MFSCommunication.h"
 
-#if MFS_ROOT_ID != FUSE_ROOT_ID
-#error FUSE_ROOT_ID is not equal to MFS_ROOT_ID
+#if SPECIAL_INODE_ROOT != FUSE_ROOT_ID
+#error FUSE_ROOT_ID is not equal to SPECIAL_INODE_ROOT
 #endif
 
 #define READDIR_BUFFSIZE 50000
@@ -49,11 +52,17 @@ void checkTypesEqual(const A& a, const B& b) {
  */
 LizardClient::Context get_context(fuse_req_t& req) {
 	auto fuse_ctx = fuse_req_ctx(req);
-	auto ret = LizardClient::Context(fuse_ctx->uid, fuse_ctx->gid, fuse_ctx->pid, fuse_ctx->umask);
+#if (FUSE_VERSION >= 28)
+	mode_t umask = fuse_ctx->umask;
+#else
+	mode_t umask = 0000;
+#endif
+	auto ret = LizardClient::Context(fuse_ctx->uid, fuse_ctx->gid, fuse_ctx->pid, umask);
 	checkTypesEqual(ret.uid,   fuse_ctx->uid);
 	checkTypesEqual(ret.gid,   fuse_ctx->gid);
 	checkTypesEqual(ret.pid,   fuse_ctx->pid);
-	checkTypesEqual(ret.umask, fuse_ctx->umask);
+	checkTypesEqual(ret.umask, umask);
+
 	return ret;
 }
 
@@ -68,7 +77,8 @@ class fuse_file_info_wrapper {
 public:
 	fuse_file_info_wrapper(fuse_file_info* fi)
 			: fuse_fi_(fi), fs_fi_(fuse_fi_
-					? new LizardClient::FileInfo(fi->flags, fi->direct_io, fi->keep_cache, fi->fh)
+					? new LizardClient::FileInfo(fi->flags, fi->direct_io, fi->keep_cache, fi->fh,
+					fi->lock_owner)
 					: nullptr) {
 	}
 	operator LizardClient::FileInfo*() {
@@ -127,6 +137,9 @@ fuse_entry_param make_fuse_entry_param(const LizardClient::EntryParam& e) {
 
 static_assert(std::is_same<LizardClient::Inode, fuse_ino_t>::value, "Types don't match");
 
+
+ThreadSafeMap<std::uintptr_t, lzfs_locks::InterruptData> gLockInterruptData;
+
 #if FUSE_USE_VERSION >= 26
 void mfs_statfs(fuse_req_t req,fuse_ino_t ino) {
 #else
@@ -178,8 +191,10 @@ void mfs_setattr(fuse_req_t req, fuse_ino_t ino, struct stat *stbuf, int to_set,
 		static_assert(LIZARDFS_SET_ATTR_SIZE      == FUSE_SET_ATTR_SIZE,      "incompatible");
 		static_assert(LIZARDFS_SET_ATTR_ATIME     == FUSE_SET_ATTR_ATIME,     "incompatible");
 		static_assert(LIZARDFS_SET_ATTR_MTIME     == FUSE_SET_ATTR_MTIME,     "incompatible");
+#if (FUSE_VERSION >= 28)
 		static_assert(LIZARDFS_SET_ATTR_ATIME_NOW == FUSE_SET_ATTR_ATIME_NOW, "incompatible");
 		static_assert(LIZARDFS_SET_ATTR_MTIME_NOW == FUSE_SET_ATTR_MTIME_NOW, "incompatible");
+#endif
 
 		auto a = LizardClient::setattr(
 				get_context(req), ino, stbuf, to_set, fuse_file_info_wrapper(fi));
@@ -463,8 +478,139 @@ void mfs_removexattr (fuse_req_t req, fuse_ino_t ino, const char *name) {
 void mfs_init(int debug_mode_, int keep_cache_, double direntry_cache_timeout_,
 		double entry_cache_timeout_, double attr_cache_timeout_, int mkdir_copy_sgid_,
 		SugidClearMode sugid_clear_mode_, bool acl_enabled_, double acl_cache_timeout_,
-		unsigned acl_cache_size_) {
+		unsigned acl_cache_size_, bool use_rwlock_) {
 	LizardClient::init(debug_mode_, keep_cache_, direntry_cache_timeout_, entry_cache_timeout_,
-			attr_cache_timeout_, mkdir_copy_sgid_, sugid_clear_mode_, acl_enabled_,
+			attr_cache_timeout_, mkdir_copy_sgid_, sugid_clear_mode_, acl_enabled_, use_rwlock_,
 			acl_cache_timeout_, acl_cache_size_);
 }
+
+#if FUSE_VERSION >= 26
+void lzfs_flock_interrupt(fuse_req_t req, void *data) {
+	auto interrupt_data = gLockInterruptData.take(reinterpret_cast<std::uintptr_t>(data));
+
+	// if there was any data
+	if (interrupt_data.first) {
+		// handle interrupt
+		LizardClient::flock_interrupt(interrupt_data.second);
+		fuse_reply_err(req, EINTR);
+	}
+}
+
+void lzfs_setlk_interrupt(fuse_req_t req, void *data) {
+	auto interrupt_data = gLockInterruptData.take(reinterpret_cast<std::uintptr_t>(data));
+
+	// if there was any data
+	if (interrupt_data.first) {
+		// handle interrupt
+		LizardClient::setlk_interrupt(interrupt_data.second);
+		fuse_reply_err(req, EINTR);
+	}
+}
+
+void lzfs_getlk(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi, struct flock *lock) {
+	try {
+		if (!lzfs_locks::posixOpValid(lock->l_type)) {
+			fuse_reply_err(req, EINVAL);
+			return;
+		}
+
+		lzfs_locks::FlockWrapper lzfslock = lzfs_locks::convertPLock(*lock, 1);
+		LizardClient::getlk(get_context(req), ino, fuse_file_info_wrapper(fi), lzfslock);
+		struct flock retlock = lzfs_locks::convertToFlock(lzfslock);
+		fuse_reply_lock(req, &retlock);
+	} catch (LizardClient::RequestException& e) {
+		fuse_reply_err(req, e.errNo);
+	}
+}
+
+
+void lzfs_setlk(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi, struct flock *lock, int sleep) {
+	std::uintptr_t interrupt_data_key = gLockInterruptData.generateKey();
+	try {
+		if (fuse_req_interrupted(req)) {
+			fuse_reply_err(req, EINTR);
+			return;
+		}
+
+		if (!lzfs_locks::posixOpValid(lock->l_type)) {
+			fuse_reply_err(req, EINVAL);
+			return;
+		}
+
+		lzfs_locks::FlockWrapper lzfslock = lzfs_locks::convertPLock(*lock, sleep);
+		uint32_t reqid = LizardClient::setlk_send(get_context(req), ino, fuse_file_info_wrapper(fi),
+				lzfslock);
+
+		// save interrupt data in static structure
+		gLockInterruptData.put(interrupt_data_key,
+				       lzfs_locks::InterruptData(fi->lock_owner, ino, reqid));
+
+		// register interrupt handle
+		if (lzfslock.l_type == lzfs_locks::kShared || lzfslock.l_type == lzfs_locks::kExclusive) {
+			fuse_req_interrupt_func(req, lzfs_setlk_interrupt,
+					reinterpret_cast<void*>(interrupt_data_key));
+		}
+
+		// WARNING: csetlk_recv() won't work with polonaise server,
+		// since actual code requires setlk_send()
+		// to be executed by the same thread.
+		LizardClient::setlk_recv();
+
+		// release the memory
+		auto interrupt_data = gLockInterruptData.take(interrupt_data_key);
+		if (interrupt_data.first) {
+			fuse_reply_err(req, 0);
+		}
+	} catch (LizardClient::RequestException& e) {
+		// release the memory
+		auto interrupt_data = gLockInterruptData.take(interrupt_data_key);
+		if (interrupt_data.first) {
+			fuse_reply_err(req, e.errNo);
+		}
+	}
+}
+#endif
+#if FUSE_VERSION >= 29
+
+void lzfs_flock(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi, int op) {
+	std::uintptr_t interrupt_data_key = gLockInterruptData.generateKey();
+	try {
+		if (fuse_req_interrupted(req)) {
+			fuse_reply_err(req, EINTR);
+			return;
+		}
+
+		if (!lzfs_locks::flockOpValid(op)) {
+			fuse_reply_err(req, EINVAL);
+			return;
+		}
+
+		uint32_t lzfs_op = lzfs_locks::flockOpConv(op);
+		uint32_t reqid = LizardClient::flock_send(get_context(req), ino,
+			fuse_file_info_wrapper(fi), lzfs_op);
+
+		// save interrupt data in static structure
+		gLockInterruptData.put(interrupt_data_key,
+				       lzfs_locks::InterruptData(fi->lock_owner, ino, reqid));
+		// register interrupt handle
+		if (lzfs_op == lzfs_locks::kShared || lzfs_op == lzfs_locks::kExclusive) {
+			fuse_req_interrupt_func(req, lzfs_flock_interrupt,
+					reinterpret_cast<void*>(interrupt_data_key));
+		}
+
+		LizardClient::flock_recv();
+
+		// release the memory
+		auto interrupt_data = gLockInterruptData.take(interrupt_data_key);
+		if (interrupt_data.first) {
+			fuse_reply_err(req, 0);
+		}
+	} catch (LizardClient::RequestException& e) {
+		// release the memory
+		auto interrupt_data = gLockInterruptData.take(interrupt_data_key);
+		if (interrupt_data.first) {
+			fuse_reply_err(req, e.errNo);
+		}
+	}
+}
+#endif
