@@ -12,11 +12,8 @@ lizardfs-polonaise-server --master-host=localhost \
 sleep 3
 mnt="$TEMP_DIR/mfspolon"
 mkdir -p "$mnt"
-
-# fsname below is important. When the test is ended framework unmounts all the filesystems
-# that match a given regex.
-polonaise-fuse-client "$mnt" -o big_writes,allow_other,fsname=mfspolon &
-MESSAGE="Client is not available" assert_eventually 'mfsdirinfo "$mnt"'
+polonaise-fuse-client "$mnt" -o big_writes,allow_other &
+MESSAGE="Client is not available" assert_eventually 'lizardfs dirinfo "$mnt"'
 
 cd "$mnt"
 for generator in $(metadata_get_all_generators | egrep -v "acl|xattr|trash"); do
