@@ -1,4 +1,4 @@
-timeout_set '4 minutes'
+timeout_set '5 minutes'
 
 assert_program_installed git
 assert_program_installed cmake
@@ -18,11 +18,8 @@ lizardfs-polonaise-server \
 sleep 3
 mnt="$TEMP_DIR/mfspolon"
 mkdir -p "$mnt"
-
-# fsname below is important. When the test is ended framework unmounts all the filesystems
-# that match a given regex.
-polonaise-fuse-client "$mnt" -o big_writes,allow_other,fsname=mfspolon &
-assert_eventually 'mfsdirinfo "$mnt"'
+polonaise-fuse-client "$mnt" -o big_writes,allow_other &
+assert_eventually 'lizardfs dirinfo "$mnt"'
 
 # Perform a compilation
 cd "$mnt"
